;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Complex Envelope follower
;
; Description : Getting the envelope of a signal based on an attack and a release time
;                 
; Nb In       : 1
; Nb Out      : 1
;
; Input       : audio signal
; Output      : envelope of the signal
;
; Cycles      : 11 + 10 cycles (-1 = unknown)
; Memory      : 3 words (without init)
; Delay Line  :	no
;
; Author      : Lall
; Date        : 13/09/2003
; Version     : 1.0
;
; Control parameters (see SEC1): Ga and Gr
;	Ga = attack constant given by  Ga = exp(-1 / (SampleRate * attack_in_sec))
;	Gr = release constant given by Gr = exp(-1 / (SampleRate * release_in_sec))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS	INIT		0x3ff		; init params at first run

	;; SEC1 Control Parameters
ABS	Ga			0x000
ABS	Gr			0x001

	;; SEC2 Initial values of control parameters (S3.24 format)
IGa			EQU	0xfd5732		;  float ga = (float) exp(-1/(SampleRate*attack)); 
IGr			EQU	0xffbb99		;  float gr = (float) exp(-1/(SampleRate*release)); 

	;; SEC3 Constants definitions
;none

	;; SEC4 Memory variables definition
ABS	LastEnv		0x002

	;; SEC5 Initial values of memory variables
ILastEnv	EQU	0.0

	;; SEC6 Initialization : (should restore register to 0x0)
CM	0x40000 	INIT
SKIP	!Z		EndInit
C		ILastEnv			; load ILastEnv
SCA 	0.0  	LastEnv		; save to LastEnv
C		IGa					; load IGa
SCA 	0.0  	Ga			; save to Ga
C		IGr					; load IGa
SCA 	0.0  	Gr			; save to Gr
C      0x1000000    ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Main
;EnvIn = abs(input); 
CM		1.0		IN1		; A contains the input
SKIP	!N		1		; if the number is positive don't do next line
LCA		-1.0	LastEnv	; invert A if number if negative
X1AC	0.0				; load A in B
; at this point, A and B contains AbsIn = abs(input)

;if(LastEnv < AbsIn)
; test enveloppe via LastEnv - AbsIn
CBM		-1.0 	LastEnv		; A = LastEnv - AbsIn
SKIP	!N		EndAttack	; skip the rest if negative number
; Compute the attack if here
AMB				Ga			; A = (LastEnv - AbsIn) * Ga + AbsIn
SKIP	EndRelease			; Go to release
EndAttack:

; Compute the release if here
;CBM	-1.0	LastEnv		; A = LastEnv - AbsIn					=> not needed as if coming here, A = LastEnv - AbsIn
;SKIP	N		EndRelease	; skip the rest if positive number		=> not needed as we now when here that release should be computed
AMB				Gr			; A = (LastEnv - AbsIn) * Gr + AbsIn
EndRelease:

SCA		1.0		LastEnv		; store the last envelope 
SCA		0.0		OUT1		; store the output

