;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Exponential interpolation
;
; Description : Interpolates in an exponential fashion a value between two extremes
;                 based on a given ratio between delay min and delay max
; Nb In       : 1
; Nb Out      : 1
;
; Input       : 1 channel signal, interpolation ratio between dMin and dMax
; Output      : 1 channel signal, the interpolated value
;
; Cycles      : 10 + 8 cycles (-1=unknownw)
; Memory      : 2 words (without init)
; Delay Line  :	no
;
; Author      : Lall
; Date        : 23/03/2003
; Version     : 1.0
;
; Control parameters (see SEC1): dMin and dMax
;	dMin: minumum "delay" for the phaser, given as normalized frequency i.e. dMin = fMin / (Fs/2)
;	dMax: maximum "delay" for the phaser, given as normalized frequency i.e. dMax = fMax / (Fs/2)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS		INIT		0x3ff 	; init params at first run

	;; SEC1 Control Parameters
ABS		dMin		0		; Minimum delay
ABS		dMax		1		; Maximum delay


	;; SEC2 Initial values of control parameters (S3.24 format)
IdMin		EQU 	0.02	; 480 Hz  = min frequency
IdMax		EQU		0.1		; 2400 Hz = max frequency

	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition

	;; SEC5 Initial values of memory variables

	;; SEC6 Initialization : (should restore register to 0x0)
CM	0x40000 INIT
SKIP	!Z	EndInit
C	IdMin			; load init value for IdMin ...
SCA	0x0 dMin		; ... Stored in dMin
C	IdMax			; load init value for IdMax ...
SCA	0x0 dMax		; ... Stored in dMax
C       0x1000000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Main

;;; Interpolate the value
1MC		1.0		IN1					; A = (triangle(n) + 1.0)
CAD		0.25	0.0					; A = (triangle(n) + 1.0) / 4
X1AC	0.0							; B = (triangle(n) + 1.0) / 4
EXPB								; A = exp16(B)
1AC		-1.0						; A = exp16 - 1.0
XCM		0.3334	dMax				; B = exp16 - 1.0		A = dMax / 3.0
CMA		-0.3334	dMin				; A = (dMax - dMin) / 3.0
BAC		0.0							; A = (dMax - dMin) * ratio / 3.0
CAM		1.0		dMin				; A = dMin + (dMax - dMin) * ratio / 3.0
SCA		1.0		OUT1				; OUT1 = dMin + (dMax - dMin) * ratio where ratio = [0.0 ... 1.0]
