;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Logarithmic interpolation
;
; Description : Interpolates logarithmically a signal between 0x0000001 and 1.0
;                 to an output between min and max
;		When signal=0, min is returned
; Nb In       : 1
; Nb Out      : 1
;
; Input       : 1 channel signal between 0 and 1
; Output      : 1 channel signal interpolated between min and max
;
; Cycles      : 11 + 8 cycles (-1=unknownw)
; Memory      : 2 words (without init)
; Delay Line  :	no
;
; Author      : Gregor
; Date        : 31/12/2003
; Version     : 1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS	INIT	0x3ff		; init params at first run

	;; SEC1 Control Parameters
ABS	Min	0x0		; Minimum
ABS	Max	0x1		; Maximum


	;; SEC2 Initial values of control parameters (S3.24 format)
IMin	EQU 	0.01		;
IMax	EQU	1.0		;

	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition

	;; SEC5 Initial values of memory variables

	;; SEC6 Initialization : (should restore register to 0x0)
CM	0x40000 INIT
SKIP	!Z	EndInit
C	IMin			; load init value for IdMin ...
SCA	0x0 Min			; ... Stored in dMin
C	IMax			; load init value for IdMax ...
SCA	0x0 Max			; ... Stored in dMax
C       0x1000000		; 1->A
SCA     0x0 INIT		; do not reinitialize next time
EndInit:

	;; SEC7 Main
	;; log16(in) -> [-6,0]
	;; out = min + (max-min) * (log16(in)+6)/6
CM	0x40000 IN1		; load input
SKIP	!Z NonZero		; if IN=0, put 0x1
C	0x1
NonZero:
XCM	0x0 IN1			; IN->B
LOGB
CAD	0x0AAAA 0x100		; rescale the log between 0 and 1
XCM	0x40000 Max		; log(IN)->B, Max->A
CMA	0x3c0000 Min		; Max-Min -> A
BAC	0x0
CAM	0x40000 Min		; Min + (Max-Min)*log
SCA	0x0 OUT1		; write output
