/////////////////////////////////////////////////////////////////////////////
// Name:        glmodule.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     06/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#include "glmodulelist.h"
#include "link.h"
#include "../AL3101ModuleHeader.h"

#include "gldefs.h"

//----------------------------------
// CglModule List Implementation
//----------------------------------

CglModuleList::CglModuleList(wxWindow *i_parent, CLinkList *i_LLink, wxGauge *i_dspg, wxGauge *i_memg){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering constructor\n");
#endif
  // parent structure is he following:
  // main frame -> total split -> upper split -> gl struct -> gl mod list
  wxFrame *MainFrame=(wxFrame*) i_parent->GetParent()->GetParent()->GetParent();

  Parent = i_parent;
  StatusBar = MainFrame->GetStatusBar();
  DSPStatus = i_dspg;
  MemStatus = i_memg;
  FillInfo.Cycles = 0;
  FillInfo.MemUsage = FillInfo.UseDelay = 0;
  FillInfo.InitCycles = 4;
  List=NULL;
  MovingModule=NULL;
  NbModules=0;
  LLink=i_LLink;
  IsModified = 0;

  // TODO: read groups from project file !!
  mpgroups = NULL;
}

CglModuleList::~CglModuleList(){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering destructor\n");
#endif

  TglModuleList *Tmp;

  while (List!=NULL){
    Tmp=List;
    delete Tmp->Mod;
    List=(TglModuleList*) Tmp->Next;
    delete Tmp;
  };
}

// initialization
void CglModuleList::Init(TTheme *theme){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering Init\n");
#endif

  ChangeTheme(theme);

  wxLogMessage("AgALag modules list Initialization done");
}

void CglModuleList::ChangeTheme(TTheme *theme){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering ChangeTheme\n");
#endif

  TglModuleList *Tmp;

  Theme = theme;

  Tmp=List;
  while (Tmp!=NULL){
    Tmp->Mod->UpdateTextures();
    Tmp=(TglModuleList*) Tmp->Next;
  };
}

// add a module
void CglModuleList::Add(wxString i_Name, wxString i_Path, wxString i_Label, int x_pos, int y_pos){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering Add\n");
#endif

  wxString mName;
  TModuleInfo info;
  CConfig *cfg;
  cfg = new CConfig;

  wxString lm, sm;
  lm << "Adding module '" << i_Name.c_str() << "(" << i_Label.c_str() << ")" << "' to OpenGL structure";
  wxLogMessage(lm);
  
  // if we ask anything else than input or output, we need to go and check the file
  if (strcmp(i_Name.c_str(), "Input") == 0){
    strcpy(info.Name, "Input");
    strcpy(info.Descr, "Incoming sound (before effects chain)");
    strcpy(info.InDescr, "<none>");
    strcpy(info.OutDescr, "8 AL3101 Sound channels");
    info.NbIn = 0;
    info.NbOut = 8;
    info.Cycles = info.MemUsage = info.UseDelay = 0;
    info.InitCycles = 4;
  } else if (strcmp(i_Name.c_str(), "Output") == 0){
    strcpy(info.Name, "Output");
    strcpy(info.Descr, "Outgoing sound (after effects chain)");
    strcpy(info.InDescr, "8 AL3101 Sound channles");
    strcpy(info.OutDescr, "<none>");
    info.NbIn = 8;
    info.NbOut = 0;
    info.Cycles = info.MemUsage = info.UseDelay = 0;
    info.InitCycles = 4;
  } else {
    char FullFName[512];
    sprintf(FullFName, "%s/%s%s.asm", cfg->EffectsDir, i_Path.c_str(),i_Name.c_str());

    // retreive module informations: number of IO's
    if (GetModuleInfo(FullFName, &info)==MODULE_HEADER_ERROR){
      char str[512];
      sprintf(str, "Module '%s' is not AgALag-compliant.", FullFName);
      wxLogMessage(str);
      wxMessageBox(str, "Error");
    }

    IsModified=1;
  }


  CglModule *Module = new CglModule(Parent, i_Name, i_Path, i_Label, Theme, 
				    x_pos, y_pos, &info);
  TglModuleList *Tmp = new TglModuleList;
  Tmp->Index=NbModules;
  Tmp->Next=(void*) List;
  List=Tmp;
  List->Mod=Module;

  // retreive module properties
  if ((strcmp(i_Name.c_str(), "Input")  != 0) && 
      (strcmp(i_Name.c_str(), "Output") != 0)){
    Module->RetreivePropertyList(cfg->EffectsDir);
    Module->BuildParamList();
  }

  NbModules++;
  FillInfo.Cycles+=info.Cycles;
  FillInfo.InitCycles+=info.InitCycles-4;
  FillInfo.MemUsage+=info.MemUsage;
  FillInfo.UseDelay+=info.UseDelay;
 
  sm << "Cycles: " << FillInfo.Cycles << "/1024 (" << FillInfo.Cycles+FillInfo.InitCycles << 
    ")    Memory: " <<FillInfo.MemUsage << "/1024     Delay: " << 
    ((FillInfo.UseDelay) ? "yes" : "no");
  DSPStatus->SetValue(FillInfo.Cycles+FillInfo.InitCycles);
  MemStatus->SetValue(FillInfo.MemUsage);

  StatusBar->SetStatusText(sm, 3);

  delete cfg;
}

void CglModuleList::Reset(){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering Reset\n");
#endif

  TglModuleList *Tmp=List;

  while (Tmp!=NULL){
    List=(TglModuleList*) Tmp->Next;
    delete Tmp->Mod;
    delete Tmp;
    Tmp=List;
  };
  
  FillInfo.Cycles = 0;
  FillInfo.MemUsage = FillInfo.UseDelay = 0;
  FillInfo.InitCycles = 4;

  NbModules=0;

  IsModified=1;
}

// check if one has clicked inside a module
// this receive coordinated and MOUSE_DOWN, MOUSE_UP or MOUSE_ISDOWN
// we return 0 if no rendering has to be performed and 1 if we muist update GL structure
int CglModuleList::ProcessClick(int px, int py, int State, int Button){
  int io;
  TglModuleList *Tmp=List;
  wxString lm, sm;

  // if there is a started link, do not process right clicks
  if ((Button==MOUSE_BUTTON_RIGHT) && (LLink->IsLinkStarted())) return 0;

  if (State==MOUSE_DOWN){
    while (Tmp!=NULL){
      switch (Tmp->Mod->IsInside(px, py, &io)){
      case MODULE_INSIDE:
	if (Button==MOUSE_BUTTON_LEFT){
	  MovingModule=Tmp->Mod;
	  MovingModule->StartMove(px, py);
	  ClickMode=CLICK_MOVING;
	  return 0;
	} else {
	  ClickMode=CLICK_NONE;
	  return 1; // here, 1 means that we must construct a menu
	}
	break;
      case MODULE_INPUT:
	if (Button==MOUSE_BUTTON_LEFT){
	  switch (LLink->Process(Tmp->Mod, io, LINK_MODE_I)){
	  case LINK_DONE:
	    ClickMode=CLICK_NONE;
	    IsModified=1;
	    return 1;
	    break;
	  case LINK_OK:
	    ClickMode=CLICK_LINKING;
	    return 0;
	    break;
	  case LINK_ERROR:
	    ClickMode=CLICK_NONE;
	    return 0;
	    break;
	  };
	} else {
	  ClickMode=CLICK_NONE;
	  return 2;
	}
	break;
      case MODULE_OUTPUT:
	if (Button==MOUSE_BUTTON_LEFT){
	  switch (LLink->Process(Tmp->Mod, io, LINK_MODE_O)){
	  case LINK_DONE:
	    ClickMode=CLICK_NONE;
	    IsModified=1;
	    return 1;
	    break;
	  case LINK_OK:
	    ClickMode=CLICK_LINKING;
	    return 0;
	    break;
	  case LINK_ERROR:
	    ClickMode=CLICK_NONE;
	    return 0;
	    break;
	  };
	} else {
	  ClickMode=CLICK_NONE;
	  return 3;
	}
	break;
      case MODULE_OUTSIDE:
	ClickMode=CLICK_NONE;
	// nothing to to 
	break;
      };
      Tmp=(TglModuleList*) Tmp->Next;
    };
    if (LLink->Process(NULL, 0, LINK_MODE_I)==LINK_DONE)
      return 1;
    
  } else if (State==MOUSE_UP){    
    if (MovingModule!=NULL){
      MovingModule->FinishMove();
      MovingModule=NULL;
      IsModified=1;
      return 1;
    }

    if (Button==MOUSE_BUTTON_RIGHT){
      // if we clicked right button, delete the selected object
      TglModuleList *Prev;
      Prev=NULL;
      while (Tmp!=NULL){
	switch (Tmp->Mod->IsInside(px, py, &io)){
	case MODULE_INSIDE:
	  if ((strcmp(Tmp->Mod->GetName().c_str(),"Input")==0)
	      || (strcmp(Tmp->Mod->GetName().c_str(),"Output")==0)){
	    wxLogMessage("Deleting module 'Input' or 'Output' is forbidden");
	    return 0;
	  }
	  if (Prev==NULL){
	    List=(TglModuleList*) Tmp->Next;
	  } else {
	    Prev->Next=Tmp->Next;
	  }
	  LLink->DeleteModuleLinks(Tmp->Mod);
	  lm << "Deleting module '" << Tmp->Mod->GetName().c_str() << "'";
	  wxLogMessage(lm);
	  
	  FillInfo.Cycles-=Tmp->Mod->info.Cycles;
	  FillInfo.InitCycles-=(Tmp->Mod->info.InitCycles-4);
	  FillInfo.MemUsage-=Tmp->Mod->info.MemUsage;
	  FillInfo.UseDelay-=Tmp->Mod->info.UseDelay;
	  
	  delete Tmp;
	  sm << "Cycles: " << FillInfo.Cycles << "/1024 (" << FillInfo.Cycles+FillInfo.InitCycles << 
	    ")    Memory: " <<FillInfo.MemUsage << "/1024     Delay: " << 
	    ((FillInfo.UseDelay) ? "yes" : "no");
	  StatusBar->SetStatusText(sm, 3);
	  DSPStatus->SetValue(FillInfo.Cycles+FillInfo.InitCycles);
	  MemStatus->SetValue(FillInfo.MemUsage);
	  IsModified=1;
	  return 1;
	  break;
	case MODULE_INPUT:
	  if (LLink->Process(Tmp->Mod, io, LINK_MODE_DELETE_I)==LINK_DONE){
	    IsModified=1;
	    return 1;
	  } else
	    return 0;
	  break;
	case MODULE_OUTPUT:
	  if (LLink->Process(Tmp->Mod, io, LINK_MODE_DELETE_O)==LINK_DONE){
	    IsModified=1;
	    return 1;
	  } else 
	    return 0;
	  break;
	case MODULE_OUTSIDE:
	  // nothing to to 
	  break;
	};
	Prev=Tmp;
	Tmp=(TglModuleList*) Tmp->Next;
      };
    }
  } else if (State==MOUSE_ISDOWN){
    if ((MovingModule!=NULL) && (MovingModule->MousePositionChanged(px, py))){
      return 1;
    }
  }

  return 0;
}

int CglModuleList::ProcessMovement(int px, int py){
  if (ClickMode!=CLICK_LINKING) return 0;

  // check if we have moved
  // if moved ! Update position and ask to redraw
  return LLink->SetActiveLinkEnd(px,py);

  return 0;
}

int CglModuleList::ProcessProp(int px, int py){
  int io;
  TglModuleList *Tmp=List;

  while (Tmp!=NULL){
    switch (Tmp->Mod->IsInside(px, py, &io)){
    case MODULE_INSIDE:
      wxString lm;
      lm  << "Updating properties of module '" << Tmp->Mod->GetName().c_str() << "'";
      wxLogMessage(lm);
      if (Tmp->Mod->UpdateProperties())
	IsModified=1;
      return 1;
      break;
    }
    Tmp=(TglModuleList*) Tmp->Next;
  };
  return 0;
}

int CglModuleList::ProcessName(int px, int py){
  int io;
  TglModuleList *Tmp=List;
  wxString TmpLabel;
  int i;
  char c;
  bool accept;

  while (Tmp!=NULL){
    switch (Tmp->Mod->IsInside(px, py, &io)){
    case MODULE_INSIDE:
      // check if the module is not the inout or output module
      if ((strcmp(Tmp->Mod->Label, "Output")!=0) && (strcmp(Tmp->Mod->Label, "Input")!=0)){
	wxTextEntryDialog NewLabelDlg(Parent, "Enter the new name", 
				      "Module properties", 
				      Tmp->Mod->Label, 
				      wxOK | wxCANCEL | wxCENTRE, 
				      wxDefaultPosition);
	if (NewLabelDlg.ShowModal() == wxID_OK){
	  TmpLabel = NewLabelDlg.GetValue();
	  // check that string only contains alphabetical and numerical characters
	  accept = true;
	  for (i=0;i<TmpLabel.Length();i++){
	    c = TmpLabel.GetChar(i);
	    c = toupper(c);
	    if (((c<'0') || ((c>'9') && (c<'A')) || (c>'Z')) && (c!='_'))
	      accept = false;
	  } 
	  if (accept){
	    Tmp->Mod->Label = TmpLabel;
	    Tmp->Mod->UpdateSize();
	    IsModified=1;
	    return 1;
	  } else {
	    wxMessageBox("Names may only contain alphabetical\nand numerical characters", "Error"); 
	    return 0;
	  }
	}
	return 0;
      } else 
	return 0;
    }
    Tmp=(TglModuleList*) Tmp->Next;
  };
  return 0;
}

int CglModuleList::ProcessInfo(int px, int py){
  int io;
  TglModuleList *Tmp=List;
  CglModule *m;

  while (Tmp!=NULL){
    switch (Tmp->Mod->IsInside(px, py, &io)){
    case MODULE_INSIDE:
      // check if the module is not the inout or output module
      if ((strcmp(Tmp->Mod->Label, "Output")!=0) && (strcmp(Tmp->Mod->Label, "Input")!=0)){
	wxString lm;
	m = Tmp->Mod;
	
	lm << "Module: " << m->Label
	   << "\n    Type: " << m->info.Name
	   << "\n    Description: " << m->info.Descr
	   << "\n    " << m->info.NbIn << " inputs, " << m->info.NbOut << " outputs"
	   << "\n    Inputs: " << m->info.InDescr
	   << "\n    Outputs: " << m->info.OutDescr;
	wxMessageBox(lm, "Information");
      } else 
	return 0;
    }
    Tmp=(TglModuleList*) Tmp->Next;
  };
  return 0;
}

int CglModuleList::ProcessFeedback(int px, int py, int mode){
  int ch;
  TglModuleList *Tmp=List;
  CglModule *m;

  while (Tmp!=NULL){
    switch (Tmp->Mod->IsInside(px, py, &ch)){
    case MODULE_INPUT:
      if (mode==LINK_MODE_FEEDBACK_I){
	LLink->Process(Tmp->Mod, ch, mode);
	IsModified=1;
	return 0;
      }
      break;
    case MODULE_OUTPUT:
      if (mode==LINK_MODE_FEEDBACK_O){
	LLink->Process(Tmp->Mod, ch, mode);
	IsModified=1;
	return 0;
      }
      break;
    }
    Tmp=(TglModuleList*) Tmp->Next;
  };
  return 0;
}


// renderizer
void CglModuleList::Render(wxDC *dc, int UseTextures){
  TglModuleList *Tmp=List;
  //  int NbX, NbY;

  //wxSize size = Parent->GetSize();
  //NbX=size.GetWidth()/SIZE_OF_GRID;
  //NbY=size.GetHeight()/SIZE_OF_GRID;

  while (Tmp!=NULL){
    Tmp->Mod->Render(dc, UseTextures);
    Tmp=(TglModuleList*) Tmp->Next;
  };

  LLink->Render(dc, UseTextures);

}

// create the module section of the script
int CglModuleList::CreateModuleSection(FILE *script){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering CreateModuleSection\n");
#endif

  TglModuleList *Tmp=List;
  int i;
  char FullName[255];
  CConfig *cfg;
  cfg = new CConfig;
  
  i=1;
  while (Tmp!=NULL){
    Tmp->Mod->SetIndex(i);
    Tmp->Index = i;
    i++;
    Tmp=(TglModuleList*) Tmp->Next;
  };
  i--;

  // i now contain the numbver of modules
  fprintf(script,"%d\n", i);
  
  wxLogMessage("Saving module list.");
  Tmp=List;
  while (Tmp!=NULL){
    strcpy(FullName, Tmp->Mod->GetName().c_str());
    if ((strcmp(FullName, "Input")!=0) && (strcmp(FullName, "Output")!=0))
      fprintf(script, "%d. %s/%s%s.asm | %s (%d-%d)\n", Tmp->Mod->GetIndex(), 
	      cfg->EffectsDir, Tmp->Mod->GetPath().c_str(), FullName, 
	      Tmp->Mod->Label.c_str(),
	      Tmp->Mod->x, Tmp->Mod->y);
    else if (strcmp(FullName, "Input")==0)
      fprintf(script, "%d. IN | %s (%d-%d)\n", Tmp->Mod->GetIndex(),  
	      Tmp->Mod->Label.c_str(),
	      Tmp->Mod->x, Tmp->Mod->y);
    else 
      fprintf(script, "%d. OUT | %s (%d-%d)\n", Tmp->Mod->GetIndex(),  
	      Tmp->Mod->Label.c_str(),
	      Tmp->Mod->x, Tmp->Mod->y);
    Tmp=(TglModuleList*) Tmp->Next;
  };

  fprintf(script,"\n");

  delete cfg;
  return 0;
}

int CglModuleList::CreatePropertiesSection(FILE *script){
  TglModuleList *Tmp;
  char FullName[255];

  Tmp=List;
  while (Tmp!=NULL){
    strcpy(FullName, Tmp->Mod->GetName().c_str());
    if ((strcmp(FullName, "Input")!=0) && (strcmp(FullName, "Output")!=0))
      Tmp->Mod->SaveProperties(script);
    Tmp=(TglModuleList*) Tmp->Next;
  };

  fprintf(script,"\n");

  return 0;
}

// read the module section of the script
int CglModuleList::ReadModuleSection(FILE *script, int offset){
#ifdef DEBUG
  fprintf(stderr, "DBG: glmodulelist: Entering ReadModuleSection\n");
#endif

  int i,j,n,l,x,y;
  bool UseNameAsLabel;
  char FullName[512],FName[255], Path[255], Label[255];

  // retreive the number of modules  
  fscanf(script, "%d", &n);
  
  wxLogMessage("Getting modules list.");
  for (i=0;i<n;i++){
    fscanf(script,"%d. %s", &j, FullName);
    if (fscanf(script," | %s", Label) !=1)
      UseNameAsLabel=true;
    else
      UseNameAsLabel=false;
    fscanf(script," (%d-%d)", &x, &y);
    if (strcmp(FullName, "IN")==0)
      Add("Input", "", "Input", x, y);
    else if (strcmp(FullName, "OUT")==0)
      Add("Output", "", "Output", x, y);
    else {
      l=j=strlen(FullName)-1;
      while ((j>=0) && (FullName[j]!='/')) j--;
      strncpy(FName, (char *)(FullName+j+1),l-j-4);
      FName[l-j-4]=0;
      if (j-offset<=0)
	sprintf(Path, "");
      else{
	strncpy(Path, (char*)(FullName+offset+1),j-offset);
	Path[j-offset]=0;
      }
      if (UseNameAsLabel)
	strcpy(Label, FName);
      Add(FName,Path,Label,x,y);
    }
  }
  return n;
}

// read the properties section
int CglModuleList::ReadPropertiesSection(FILE *script){
  int Continue,tmp;
  int i,j;
  char PValue[255];
  TglModuleList *TmpList=List;

  Continue=1;
  while ((Continue) && (!feof(script))){
    tmp=fscanf(script, "%d %d %s", &i, &j, PValue);
    if ((tmp==0) || (tmp==EOF)){
      Continue=0;
    } else {
      if (i>NbModules) return 1;
      // we've got a property, search the module;
      TmpList=List;
      for (tmp=NbModules;tmp>i;tmp--) TmpList=(TglModuleList*) TmpList->Next;
      TmpList->Mod->SetProperty(j,PValue);
    }
  };

  return 0;
}
