/////////////////////////////////////////////////////////////////////////////
// Name:        glmodule.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     06/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __PROJ_AL_GLMODULE_LIST_H__
#define __PROJ_AL_GLMODULE_LIST_H__

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/statusbr.h>
#include "../AL3101ModuleHeader.h"
#include "glmodule.h"
#include "link.h"
#include "Config.h"
#include "theme.h"

#define MOUSE_UP     0
#define MOUSE_DOWN   1
#define MOUSE_ISDOWN 2

#define CLICK_NONE    0
#define CLICK_MOVING  1
#define CLICK_LINKING 2

typedef struct{
  CglModule *Mod;
  int Index;
  void *Next;
} TglModuleList;

class CglModuleList{
 public:
  CglModuleList(wxWindow *i_parent, CLinkList*, wxGauge*, wxGauge*);
  ~CglModuleList();

  TglModuleList *List;
  TParameterItem *mpgroups;  // groups of parameters for MissParker
  TModuleInfo FillInfo;
  int IsModified;

  // initialize textures and fonts needed for modules rendering
  void Init(TTheme *theme);
  void ChangeTheme(TTheme *theme);

  // add a module
  void Add(wxString i_Name, wxString i_Path, wxString i_Label, int x, int y);

  // reset the list of modules
  void Reset();

  // check if one has clicked inside a module
  // this receive coordinated and MOUSE_DOWN or MOUSE_UP
  int ProcessClick(int px, int py, int State, int Button);

  // modify properties of a module, given its position
  int ProcessProp(int px, int py);
  int ProcessName(int px, int py);
  int ProcessMovement(int px, int py);
  int ProcessInfo(int px, int py);
  int ProcessFeedback(int px, int px, int mode);

  // draw all the modules
  void Render(wxDC *dc, int UseTextures=1);

  // create the module section of the script
  int CreateModuleSection(FILE *script);
  int CreatePropertiesSection(FILE *script);

  // read the module section of the script
  int ReadModuleSection(FILE *script, int);
  int ReadPropertiesSection(FILE *script);

 private:
  int NbModules;
  TglModuleList *Current;
  CglModule *MovingModule;
  CLinkList *LLink;
  /* click mode can be 
  ** CLICK_NONE
  ** CLICK_MOVING
  ** CLICK_LINKING
  */
  int ClickMode;

  wxGauge *DSPStatus;
  wxGauge *MemStatus;

  wxWindow *Parent;
  // status bar nneded to report memory and code fill
  wxStatusBar *StatusBar;

  TTheme *Theme;
  wxFont *Font;
};
  
#endif
