/////////////////////////////////////////////////////////////////////////////
// Name:        glstruct.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     05/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
#pragma implementation
#pragma interface
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "glmodulelist.h"
#include "glstruct.h"
#include "gldefs.h"

#include <wx/dcmemory.h>

/*----------------------------------------------------------------
  Implementation of Test-GLCanvas
-----------------------------------------------------------------*/

#define ID_PPM_DEL    15
#define ID_PPM_PROP   16
#define ID_PPM_NAME   17
#define ID_PPM_INFO   18

#define ID_PPL_DEL    19
#define ID_PPL_FEED   20

BEGIN_EVENT_TABLE(CglStruct, wxScrolledWindow)
  EVT_SIZE(CglStruct::OnSize)
  EVT_PAINT(CglStruct::OnPaint)
  //EVT_ERASE_BACKGROUND(CglStruct::OnEraseBackground)
  //EVT_ENTER_WINDOW(CglStruct::OnEnterWindow)

  EVT_LEFT_DOWN(CglStruct::OnMouseEvent)
  EVT_LEFT_UP(CglStruct::OnMouseEvent)
  EVT_RIGHT_DOWN(CglStruct::OnMouseEvent)
  EVT_RIGHT_UP(CglStruct::OnMouseEvent)
  EVT_MIDDLE_DOWN(CglStruct::OnMouseEvent)
  EVT_MIDDLE_UP(CglStruct::OnMouseEvent)
  EVT_MOTION(CglStruct::OnMouseEvent)

  EVT_MENU(ID_PPM_DEL,  CglStruct::OnDeleteModule)
  EVT_MENU(ID_PPM_PROP, CglStruct::OnPropModule)
  EVT_MENU(ID_PPM_NAME, CglStruct::OnNameModule)
  EVT_MENU(ID_PPM_INFO, CglStruct::OnInfoModule)

  EVT_MENU(ID_PPL_DEL,  CglStruct::OnDeleteModule)
  EVT_MENU(ID_PPL_FEED, CglStruct::OnToggleFeedback)
END_EVENT_TABLE()

CglStruct::CglStruct(wxWindow *parent, TTheme *i_theme,
		     wxGauge *i_dspg, wxGauge *i_memg,
		     wxWindowID id, const wxPoint& pos, 
		     const wxSize& size, long style, const wxString& name)
  : wxScrolledWindow( parent, id, pos, size, wxSUNKEN_BORDER, name){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering constructor\n");
#endif
  
  DSPStatus = i_dspg;
  MemStatus = i_memg;
  m_init = FALSE;
  SubMenu = NULL;

  theme = i_theme;
  LLink=new CLinkList(theme);
  Modules = new CglModuleList(this, LLink, DSPStatus, MemStatus);

  // set window as a drop target
  DropTarget = new CItemDropTarget(this);
  SetDropTarget(DropTarget);
}

CglStruct::~CglStruct(){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering destructor\n");
#endif
  
  delete LLink;
  delete Modules;

  if (SubMenu!=NULL) delete SubMenu;
}

int CglStruct::InitStruct(CFile *i_project){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering InitStruct\n");
#endif
     
  Project = i_project;

  // Scrollbars
  NbX=(STRUCT_H_SIZE)/SIZE_OF_GRID;
  NbY=(STRUCT_V_SIZE)/SIZE_OF_GRID;

  SetScrollRate(SIZE_OF_GRID, SIZE_OF_GRID);
  SetVirtualSize(STRUCT_H_SIZE, STRUCT_V_SIZE);

  SubMenu = new wxMenu();
  SubMenu->Append(ID_PPM_DEL,  _T("&Delete"),     _T("Delete module"));
  SubMenu->Append(ID_PPM_PROP, _T("&Properties"), _T("Module properties"));
  SubMenu->Append(ID_PPM_NAME, _T("&Rename"),     _T("Module name"));
  SubMenu->Append(ID_PPM_INFO, _T("&Info"),       _T("Module informations"));

  LinkSubMenu = new wxMenu();
  LinkSubMenu->Append(ID_PPL_DEL,  _T("&Delete"),   _T("Delete module"));
  LinkSubMenu->Append(ID_PPL_FEED, _T("&Feedback"), _T("Toggle feedback"));
  
  return 0;
}

void CglStruct::Redraw(){
  /*int w, h;
  wxMemoryDC dbldc;
  Render(&dbldc);

  wxClientDC dc(this);
  dc.GetSize(&w,&h);
  PrepareDC(dc);
  dc.Blit(0,0,w,h,&dbldc,0,0);
  */
  Refresh();
}

void CglStruct::Render(wxDC *dc){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering Render\n");
#endif
  
  /* init OpenGL once, but after SetCurrent */
  if (!m_init){
    InitDC(dc);
    m_init = TRUE;
  }
    
  int i, j;

  // clear dc
  SetBackgroundColour(wxT("BLACK"));
  dc->SetBackground(wxBrush(wxColour(0, 0, 0), wxSOLID));
  dc->SetBackgroundMode(wxTRANSPARENT);
  dc->Clear();

  // draws the grid
  //  wxColour PtColour(255, 255, 0);
  dc->SetPen(wxPen(wxColour(255,255,0), 1, wxSOLID));
  for (i=0;i<=NbX;i++)
    for (j=0;j<=NbY;j++){
      dc->DrawPoint(i*SIZE_OF_GRID, j*SIZE_OF_GRID);
    }

  // draws input and output modules
  Modules->Render(dc);
}

void CglStruct::OnPaint( wxPaintEvent &WXUNUSED(event)){
  //int w, h;
  wxPaintDC dc(this); 
  //dc.GetSize(&w,&h);
  //wxBitmap dblbmp(w,h);
  //wxMemoryDC dbldc;
  //dbldc.SelectObject(dblbmp);
  PrepareDC(dc);
  Render(&dc);
  //dc.Blit(0,0,w,h,&dbldc,0,0);
}

void CglStruct::OnSize(wxSizeEvent& event){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering OnSize\n");
#endif
  
  SetScrollRate(SIZE_OF_GRID, SIZE_OF_GRID);
  SetVirtualSize(STRUCT_H_SIZE, STRUCT_V_SIZE);

  Redraw();
}

void CglStruct::OnMouseEvent(wxMouseEvent& event){
  int x, y, rv;
  int width, height;
  int MouseButton;
  //static wxPoint InitViewUpdate;
  wxPoint Pt=event.GetPosition();

  CalcUnscrolledPosition(Pt.x, Pt.y, &x, &y);
  x/=SIZE_OF_GRID;
  y/=SIZE_OF_GRID;

  GetClientSize(&width, &height);
  
  // we process -  left button change to down
  //            -  left                  up
  //            - right                  down

  if ((event.RightDown()) || (event.RightUp())){
    MouseButton=MOUSE_BUTTON_RIGHT;
  } else {
    MouseButton=MOUSE_BUTTON_LEFT;
  }

  if (event.ButtonDown()){
    rv = Modules->ProcessClick(x, y, MOUSE_DOWN, MouseButton);
    if (rv!=0){
      if (MouseButton==MOUSE_BUTTON_LEFT){
	Redraw();
      } else {
	tmp_px=x; tmp_py=y;
	switch (rv){
	case 1:
	  PopupMenu(SubMenu, Pt);
	  break;
	case 2:
	  tmp_io=LINK_MODE_FEEDBACK_I;
	  PopupMenu(LinkSubMenu, Pt);
	  break;
	case 3:
	  tmp_io=LINK_MODE_FEEDBACK_O;
	  PopupMenu(LinkSubMenu, Pt);
	  break;
	}
      }
    }
  } else if (event.ButtonUp()){    
    if (Modules->ProcessClick(x, y, MOUSE_UP, MouseButton)!=0){
      Redraw();
    }
  } else if (event.LeftIsDown()){
    if (Modules->ProcessClick(x, y, MOUSE_ISDOWN, MouseButton)!=0){
      Redraw();
    }
  } else {
    if (Modules->ProcessMovement(x,y)!=0){
      Redraw();
    }
  }

  event.Skip();
}

void CglStruct::InitDC(wxDC *dc){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering InitGL\n");
#endif

  Modules->Init(theme);

  // create the project file instance and initialize it
  CConfig *cfg;
  cfg=new CConfig;
  Project->Open(cfg->ProjectName);
  delete cfg;
}

void CglStruct::AddModule(char *Name, char *Path, int xCoord, int yCoord){
#ifdef DEBUG
  fprintf(stderr, "DBG: glstruct: Entering AddModule\n");
#endif

  if (xCoord==-1)
    Modules->Add(Name, Path, Name, 1, NbY/2-1);
  else{
    int x,y;
    CalcUnscrolledPosition(xCoord, yCoord, &x, &y);
    Modules->Add(Name, Path, Name, x/SIZE_OF_GRID, y/SIZE_OF_GRID);
  }
  Refresh();
}

void CglStruct::OnDeleteModule(wxCommandEvent& event){
  Modules->ProcessClick(tmp_px, tmp_py, MOUSE_UP, MOUSE_BUTTON_RIGHT);
  Redraw();
}

void CglStruct::OnPropModule(wxCommandEvent& event){
  Modules->ProcessProp(tmp_px, tmp_py);
}

void CglStruct::OnNameModule(wxCommandEvent& event){
  if (Modules->ProcessName(tmp_px, tmp_py))
    Redraw();
}

void CglStruct::OnInfoModule(wxCommandEvent& event){
  Modules->ProcessInfo(tmp_px, tmp_py);
}

void CglStruct::OnToggleFeedback(wxCommandEvent& event){
  Modules->ProcessFeedback(tmp_px, tmp_py, tmp_io);
  Redraw();
}


////////////////////////////////////
// Dropping target implementation
////////////////////////////////////

CItemDropTarget::CItemDropTarget(void *i_owner){
  owner = i_owner;
}

bool CItemDropTarget::OnDropText(wxCoord x, wxCoord y, const wxString& text){
  char FName[255], ModuleName[255], tmp[512];
  int i,l;

  strcpy(tmp, text.c_str());

  i=0; while(tmp[i]!='$') i++;
  strncpy(ModuleName, tmp, i);
  ModuleName[i]=0;
  l=strlen(tmp)-i-1;
  strncpy(FName, (char*) (tmp+i+1), l);
  FName[l]=0;

  ((CglStruct*)owner)->AddModule(ModuleName, FName, x, y);

  return TRUE;
}
