#include "mpprojectfile.h"
#include <wx/filename.h>
#include <wx/file.h>

#ifdef WIN32
#include <direct.h>
#endif

#define ERROR(s) wxLogMessage(s);
#define ERROR_2(s, p1,p2) fprintf(stderr, s, p1, p2)

TParameter_tag::TParameter_tag(){
  int i;

  ComboLabels = NULL;
  DefaultValue=-1;
  for (i=0;i<MAXSUBPARAM;i++){
    ParamTable[i]  = NULL;
    sprintf(TableFileName[i], "\0");
  }
}

TParameter_tag::~TParameter_tag(){
  int i;

  if (ComboLabels!=NULL)
    delete[] ComboLabels;

  for (i=0;i<MAXSUBPARAM;i++)
    if (ParamTable[i]!=NULL)
      delete[] ParamTable[i];
}

TParameterItem_tag::TParameterItem_tag(){
  InGroup=0;
}

int AssignDefaultValues(TParameterItem *item){
  item->NumberOfSteps = 128;
  item->DisplayType = DISPLAY_FLOAT;
  item->NumberOfSubParams = 1;
  item->ParamValue[0]=item->DisplayInitialValue=0.0;
  item->ParamMinValue[0]=item->DisplayMinValue=-1.0;
  item->ParamMaxValue[0]=item->DisplayMaxValue=1.0;

  item->ProcessType = AUTO_SLIDER_LIN;
  item->Enabled = true;

  return 0;
}

char* fgetstr(char *String, int NumberOfChar, FILE* fin){
  char *ReturnString;
  int lenstr;
  
  ReturnString = fgets(String, NumberOfChar, fin);
  lenstr = strlen(String);
#ifdef WIN32
  String[lenstr - 1] = 0;
#else
  while ((String[lenstr - 1]=='\n') || (String[lenstr - 1]=='\r')){ String[lenstr - 1]='\0'; lenstr--;};
#endif
  
  return ReturnString;
}

int LoadTable(char *filename, int *Table, int NumberOfSteps){
  FILE *flux;
  int index, ReturnCode;

  // Let's first open the table
  flux = fopen(filename, "rt");
  if (flux == NULL) return -1;
  
  for (index = 0; index < NumberOfSteps; index++){
    ReturnCode = fscanf(flux, "%i\n", &Table[index]);
    if (ReturnCode != 1) return -2;
  }

  if (fscanf(flux, "%i\n", &ReturnCode) == 1) return -3;
  
  return 0;
}

int LoadParameter(char *ParamFileName, int AddressOffset, TParameterItem **Item, bool LoadTables){
  FILE *ParamFile;
  char TempStr[FILENAME_MAX];
  char TempStr2[FILENAME_MAX];
  char TempStr3[FILENAME_MAX];
  int NumberOfSubParams;
  int aaa, aab;
  int CurSteps;
  TParameterItem *it;

  // Opening the parameter file
  ParamFile = fopen(ParamFileName, "rt");
  if (ParamFile == NULL) return -1;

  // Get the expected keyword
  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
    fclose(ParamFile);
    return -2;
  }

  // Let's now loop as long as we find valid keyword [param]
  while (strcmp(TempStr, "[param]") == 0){
    it = new TParameterItem;
    it->Enabled = true;

    // Get the parameter name
    if (fgetstr(it->ParameterName, FILENAME_MAX - 1, ParamFile) == NULL){
      fclose(ParamFile);
      return -2;
    }

    // Get the parameter description
    if (fgetstr(it->ParameterDescription, FILENAME_MAX - 1, ParamFile) == NULL){
      fclose(ParamFile);
      return -2;
    }

    // Get the kind of display and handling we should use
    if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
      fclose(ParamFile);
      return -2;
    }
    if (strcmp(TempStr, "auto") == 0){           // default processing type
      if (fgetstr(TempStr2, FILENAME_MAX - 1, ParamFile) == NULL){
	fclose(ParamFile);
	return -2;
      }
      if (strcmp(TempStr2, "slider") == 0){        // slider box
	if (fgetstr(TempStr3, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	
	// Check which kind of curve mapping to apply
	if (strcmp(TempStr3, "exp") == 0){           // exponential
	  it->ProcessType = AUTO_SLIDER_EXP;
	} else if (strcmp(TempStr3, "log") == 0){    // logarithmic
	  it->ProcessType = AUTO_SLIDER_LOG;
	} else {                                     // linear
	  it->ProcessType = AUTO_SLIDER_LIN;
	}

	// Get the number of steps
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->NumberOfSteps = atoi(TempStr);
				
	// Get the display type
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	if (strcmp(TempStr, "float") == 0){          // float
	  it->DisplayType = DISPLAY_FLOAT;
	} else {                                     // int
	  it->DisplayType = DISPLAY_INT;
	}

	// Get the values for display
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DisplayMinValue = atof(TempStr);
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DisplayMaxValue = atof(TempStr);
	if (fgetstr(TempStr3, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DisplayInitialValue = atof(TempStr3);
	
	// Get the number of "subparameter" to read
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	NumberOfSubParams = atoi(TempStr);
	if (NumberOfSubParams < 0){
	  ERROR("Negative number of sub-parameters not allowed.\n");
	  fclose(ParamFile);
	  return -2;
	}
	if (NumberOfSubParams > MAXSUBPARAM){
	  ERROR_2("Number of sub-parameters (%i) out of limit: %i.\n", NumberOfSubParams, MAXSUBPARAM);
	  fclose(ParamFile);
	  return -2;
	}
	
	for (aaa = 0; aaa < NumberOfSubParams; aaa++){
	  // Get the address of the parameter
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  it->ParamAddress[aaa] = AddressOffset + atoi(TempStr);
	  
	  // Get the parameter values 
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  it->ParamMinValue[aaa] = atof(TempStr);
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  it->ParamMaxValue[aaa] = atof(TempStr);
	  it->ParamValue[aaa] = 0.0f;
	}
	it->NumberOfSubParams = NumberOfSubParams;
      } else if (strcmp(TempStr2, "combo") == 0){      // combo box
	it->ProcessType = -1;
      } else {                                         // unknown type
	it->ProcessType = -2;
      }
    } else if (strcmp(TempStr, "table") == 0){       // table processing type
      if (fgetstr(TempStr2, FILENAME_MAX - 1, ParamFile) == NULL){
	fclose(ParamFile);
	return -2;
      }
      if (strcmp(TempStr2, "slider") == 0){            // slider box
	if (fgetstr(TempStr3, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}

	// Check which kind of curve mapping to apply
	if (strcmp(TempStr3, "exp") == 0){               // exponential
	  it->ProcessType = TABLE_SLIDER_EXP;
	} else if (strcmp(TempStr3, "log") == 0){        // logarithmic
	  it->ProcessType = TABLE_SLIDER_LOG;
	} else {                                         // linear
	  it->ProcessType = TABLE_SLIDER_LIN;
	}

	// Get the number of steps
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->NumberOfSteps = atoi(TempStr);
				
	// Get the display type
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	if (strcmp(TempStr, "float") == 0){              // float
	  it->DisplayType = DISPLAY_FLOAT;
	} else {                                         // int
	  it->DisplayType = DISPLAY_INT;
	}

	// Get the values for display
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DisplayMinValue = atof(TempStr);
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DisplayMaxValue = atof(TempStr);
	if (fgetstr(TempStr3, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DisplayInitialValue = atof(TempStr3);
	
	// Get the number of "subparameter" to read
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	NumberOfSubParams = atoi(TempStr);
	if (NumberOfSubParams < 0){
	  ERROR("Negative number of sub-parameters not allowed.\n");
	  fclose(ParamFile);
	  return -2;
	}
	if (NumberOfSubParams > MAXSUBPARAM){
	  ERROR_2("Number of sub-parameters (%i) out of limit: %i.\n", NumberOfSubParams, MAXSUBPARAM);
	  fclose(ParamFile);
	  return -2;
	}
	
	for (aaa = 0; aaa < NumberOfSubParams; aaa++){
	  // Get the address of the parameter
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  it->ParamAddress[aaa] = AddressOffset + atoi(TempStr);
	  
	  // Get the table file name 
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  sprintf(it->TableFileName[aaa], TempStr);
	  
	  if (LoadTables){
	    // Allocate the table memory
	    it->ParamTable[aaa] = new int[it->NumberOfSteps];
	    if (it->ParamTable[aaa] == NULL) return -2;
	    
	    // Load the table
	    if (LoadTable(TempStr, it->ParamTable[aaa], it->NumberOfSteps) != 0){
	      fclose(ParamFile);
	      return -2;
	    }
	  }
	}
	it->NumberOfSubParams = NumberOfSubParams;
      } else if (strcmp(TempStr2, "combo") == 0){      // combo box
	wxString DisplayValues[MAXSUBPARAM];

	// We're working with a combo box here
	it->ProcessType = TABLE_COMBO;
	
	// Get the number of steps
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->NumberOfSteps = atoi(TempStr);
	
	// Get the values for display
	it->ComboLabels=new TComboLabel[it->NumberOfSteps];
	for (aaa = 0; aaa < it->NumberOfSteps; aaa++){
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  DisplayValues[aaa] = TempStr;
	  strcpy(it->ComboLabels[aaa], TempStr);
	}

	// Get the index of the value to display
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	it->DefaultValue = CurSteps = atoi(TempStr);
	
	// Get the number of "subparameter" to read
	if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	  fclose(ParamFile);
	  return -2;
	}
	NumberOfSubParams = atoi(TempStr);
	if (NumberOfSubParams < 0){
	  ERROR("Negative number of sub-parameters not allowed.\n");
	  fclose(ParamFile);
	  return -2;
	}
	if (NumberOfSubParams > MAXSUBPARAM){
	  ERROR_2("Number of sub-parameters (%i) out of limit: %i.\n", NumberOfSubParams, MAXSUBPARAM);
	  fclose(ParamFile);
	  return -2;
	}
	
	for (aaa = 0; aaa < NumberOfSubParams; aaa++){
	  // Get the address of the parameter
	  if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	    fclose(ParamFile);
	    return -2;
	  }
	  it->ParamAddress[aaa] = AddressOffset + atoi(TempStr);

	  // Allocate some space for the parameter values
	  it->ParamTable[aaa] = new int[it->NumberOfSteps];
	  
	  // And read the values
	  for (aab = 0; aab < it->NumberOfSteps; aab++){
	    // Get the parameter values 
	    if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
	      fclose(ParamFile);
	      return -2;
	    }
	    float TempFloat = atof(TempStr) * 16777216.0f;
	    (it->ParamTable[aaa])[aab] = (int)TempFloat;
	  }
	}
	it->NumberOfSubParams = NumberOfSubParams;
      }
    }
    
    // The item is now initialized
    it->ItemInitialized = 1;
    //*index = *index + 1;					// we'll have a new parameter
    //Item = Item + 1;
    //if (*index == MAXPARAM) goto LoadParameter_ExitFunction;
    it->next = (*Item);
    (*Item) = it;
    
    // Get the expected keyword, we'll loop if necessary
    if (fgetstr(TempStr, FILENAME_MAX - 1, ParamFile) == NULL){
      fclose(ParamFile);
      return -2;
    }
  }
  
 LoadParameter_ExitFunction:
  // Closing the parameter file
  fclose(ParamFile);
  
  return 0;
}

int LoadProject(const char *ProjectFileName, TProjectParameters **Params, bool LoadTables){
  FILE *ProjectFile;
  int index;
  int AddressOffset;
  char TempStr[FILENAME_MAX];
  char TempStr2[FILENAME_MAX];
  char TempStr3[FILENAME_MAX];
  char TempStr4[FILENAME_MAX];
  wxString str;
  TProjectParameters *p;
  wxString CurrentWD;
  
  // Getting the path information out of the file name
  str = ProjectFileName;
  str = wxPathOnly(str);

  CurrentWD = wxGetWorkingDirectory();
  // Changing the current directory
  if (wxSetWorkingDirectory(str) != TRUE){
    ERROR("Unable to set the current directory.\n");
    return -1;
  }

  // Opening the parameter file
  ProjectFile = fopen(ProjectFileName, "rt");
  if (ProjectFile == NULL){
    //ERROR("Cannot open the project file\n");
    wxSetWorkingDirectory(CurrentWD);
    return -1;
  }
  
  // Check that the project file is a project file
  if (fgetstr(TempStr, FILENAME_MAX - 1, ProjectFile) == NULL){
    ERROR("Failed to check project file type\n");
    wxSetWorkingDirectory(CurrentWD);
    fclose(ProjectFile);
    return -2;
  }
  if (strcmp(TempStr, "Miss Parker Project 1.0") != 0){
    ERROR("Bad file type\n"); 
    wxSetWorkingDirectory(CurrentWD);
    fclose(ProjectFile);
    return -5;
  }
  
  // Get the effect name
  if (fgetstr(TempStr, FILENAME_MAX - 1, ProjectFile) == NULL){
    ERROR("Failed to get effect name\n");
    wxSetWorkingDirectory(CurrentWD);
    fclose(ProjectFile);
    return -2;
  }
  
  // Get the effect description
  if (fgetstr(TempStr, FILENAME_MAX - 1, ProjectFile) == NULL){
    ERROR("Failed to get effect name\n");
    wxSetWorkingDirectory(CurrentWD);
    fclose(ProjectFile);
    return -2;
  }
  
  // Get the effect map (or 'nomap' if no map present)
  if (fgetstr(TempStr, FILENAME_MAX - 1, ProjectFile) == NULL){
    ERROR("Failed to get effect name\n");
    wxSetWorkingDirectory(CurrentWD);
    fclose(ProjectFile);
    return -2;
  }
  
  (*Params)=NULL;
  //for (index = 0; index < MAXPARAM; index++) Items[index].ItemInitialized = 0;
  
  // Now get the parameters one by one
  index = 0;
  while(1){		
    if (fgetstr(TempStr, FILENAME_MAX - 1, ProjectFile) == NULL) break;
    if (fgetstr(TempStr2, FILENAME_MAX - 1, ProjectFile) == NULL) break; // read the description but don't use it for now
    if (fgetstr(TempStr3, FILENAME_MAX - 1, ProjectFile) == NULL) break;
    if (fgetstr(TempStr4, FILENAME_MAX - 1, ProjectFile) == NULL) break;
    AddressOffset = atoi(TempStr4);
    p=new TProjectParameters;
    p->next = (*Params);
    p->AddressOffset = AddressOffset;
    strcpy(p->ModName, TempStr);
    strcpy(p->ModDescr, TempStr2);
    p->param = NULL;
    (*Params) = p;
    if (LoadParameter(TempStr3, AddressOffset, &(p->param), LoadTables) != 0){
      //WARNING_1("Impossible to load complete parameter file: %s\n", TempStr);
    }
    if (index == MAXPARAM) break;
  }
  
  // Closing the parameter file
  fclose(ProjectFile);
  
  wxSetWorkingDirectory(CurrentWD);

  return 0;
}

int WriteProjectParameter(char *path, char *fname, TParameterItem *item){
  FILE *flux;
  TParameterItem *it;
  char fn[512];
  int i;

  sprintf(fn, "%s/%s", path, fname);
  flux = fopen(fn, "w+");
  if (flux==NULL)
    return 1;

  it = item;
  
  while (it!=NULL){
    if (it->Enabled){
      fprintf(flux, "[param]\n");
      fprintf(flux, "%s\n", it->ParameterName);
      fprintf(flux, "%s\n", it->ParameterDescription);
      switch (it->ProcessType){
      case AUTO_SLIDER_LIN:  fprintf(flux, "auto\nslider\nlin\n"); break;
      case AUTO_SLIDER_LOG:  fprintf(flux, "auto\nslider\nlog\n"); break;
      case AUTO_SLIDER_EXP:  fprintf(flux, "auto\nslider\nexp\n"); break;
      case TABLE_SLIDER_LIN: fprintf(flux, "table\nslider\nlin\n"); break;
      case TABLE_SLIDER_LOG: fprintf(flux, "table\nslider\nlog\n"); break;
      case TABLE_SLIDER_EXP: fprintf(flux, "table\nslider\nexp\n"); break;
      case TABLE_COMBO:      fprintf(flux, "table\ncombo\n"); break;
      };
      
      if (it->ProcessType!=TABLE_COMBO){
	fprintf(flux, "%d\n", it->NumberOfSteps);
	switch (it->DisplayType){
	case DISPLAY_INT:   
	  fprintf(flux, "int\n");
	  fprintf(flux, "%d\n", (int) (it->DisplayMinValue));
	  fprintf(flux, "%d\n", (int) (it->DisplayMaxValue));
	  fprintf(flux, "%d\n", (int) (it->DisplayInitialValue));
	  break;
	case DISPLAY_FLOAT: 
	  fprintf(flux, "float\n");
	  fprintf(flux, "%e\n", it->DisplayMinValue);
	  fprintf(flux, "%e\n", it->DisplayMaxValue);
	  fprintf(flux, "%e\n", it->DisplayInitialValue);
	  break;
	};
	fprintf(flux, "1\n");
	if ((it->ProcessType==AUTO_SLIDER_LIN) || (it->ProcessType==AUTO_SLIDER_LOG) || (it->ProcessType==AUTO_SLIDER_EXP)){
	  fprintf(flux, "%d\n", it->ParamAddress[0]);
	  fprintf(flux, "%e\n", it->ParamMinValue[0]);
	  fprintf(flux, "%e\n", it->ParamMaxValue[0]);
	} else {
	  fprintf(flux, "%d\n", it->ParamAddress[0]);
	  fprintf(flux, "%s\n", it->TableFileName);
	}
      } else { // COMBO
	fprintf(flux, "%d\n", it->NumberOfSteps);
	for (i=0;i<it->NumberOfSteps;i++)
	  fprintf(flux, "%s\n", it->ComboLabels[i]);
	// write default value
	fprintf(flux, "%d\n", (it->DefaultValue==-1) ? 0 : it->DefaultValue);
	// only 1 sub-param
	fprintf(flux, "1\n");
	fprintf(flux, "%d\n", it->ParamAddress[0]);
	for (i=0;i<it->NumberOfSteps;i++)
	  fprintf(flux, "%d\n", it->ParamTable[0][i]);
      }
    } // end of "if Enable"
    it = it->next;
  };

  fclose(flux);
  return 0;
}

// set the map variable to NULL if no map file is requested.
int WriteProject(char *ObjectFileName, char *Name, char *map, 
		 TProjectParameters *params){

  FILE *flux;
  wxFileName fn_obj(ObjectFileName);
  wxFileName fn_prj(fn_obj);
  char fname[512];
  char path[512];
  char dname[512];
  char tmp[512];
  TProjectParameters *pl;
  TParameterItem *it;
  bool enabled, creable;

  // open project file
  fn_prj.SetExt("prj");
  strcpy(fname, fn_prj.GetFullPath().c_str());
  strcpy(path, fn_prj.GetPath().c_str());
  strcpy(dname, fn_prj.GetName().c_str());
  flux=fopen(fname, "w+");

  // write project header
  fprintf(flux, "Miss Parker Project 1.0\n");
  fprintf(flux, "%s\n", Name);
  fprintf(flux, "%s\n", wxFileName(ObjectFileName).GetFullName().c_str());
  if (map==NULL)
    fprintf(flux, "nomap\n");
  else
    fprintf(flux, "%s\n", map);

  // create directory for parameter files
  sprintf(tmp, "%s/%s", path, dname);
  creable = true;
  if (wxDirExists(tmp)){
    // RmDir does only delete empty dirs ... so makes it empty first !
    wxString FileNameToKill = wxFindFirstFile(wxString(tmp)+wxString("/*.*"));
    while(!FileNameToKill.IsEmpty()){
      // check attributes
      if (!(wxFile::Access(FileNameToKill,wxFile::read)))
	creable = false;
      else {
	wxRemoveFile(FileNameToKill);
	FileNameToKill = wxFindNextFile();
      }
    };
    wxRmdir(tmp);
  }
  if (creable)
    wxMkdir(tmp);
  
  // browse all paramters
  pl = params;
  while (pl!=NULL){
    if (pl->param!=NULL){
      // check that at least of of the param is enabled
      enabled = false;
      it = pl->param; 
      while (it!=NULL){
	if (it->Enabled) enabled = true;
	it = it->next;
      };
      if (enabled){
	fprintf(flux, "%s\n", pl->ModName);
	fprintf(flux, "%s\n", pl->ModDescr);
	sprintf(fname, "%s/%s.txt", dname, pl->ModName);
	fprintf(flux, "%s\n", fname);
	fprintf(flux, "%d\n", pl->AddressOffset);
	// write parameter file
	WriteProjectParameter(path,fname,pl->param);
      }
    }
    pl = pl->next;
  }
  fclose(flux);

  return 0;
}

int DeleteParamList(TParameterItem **it){
  // delete the parameter list
  TParameterItem *il;

  while ((*it)!=NULL){
    il = (*it)->next;
    // delete the combo table
    //TODO
    delete (*it);
    (*it)=il;
  };

  return 0;
}
