#ifndef __MP_PROJECT_FILE_H__
#define __MP_PROJECT_FILE_H__

#include "wx/wx.h"
#include <wx/treectrl.h>

#define MAXPARAM		32
#define MAXSUBPARAM		5

#define POT_DO_NOTHING		0
#define AUTO_SLIDER		1
#define AUTO_SLIDER_LIN		1
#define AUTO_SLIDER_LOG		2
#define AUTO_SLIDER_EXP		3
#define TABLE_SLIDER		4
#define TABLE_SLIDER_LIN	4
#define TABLE_SLIDER_LOG	5
#define TABLE_SLIDER_EXP	6
#define TABLE_COMBO		8

#define DISPLAY_INT		0
#define DISPLAY_FLOAT	1

typedef char TComboLabel[80];
typedef char TFileName[FILENAME_MAX];

typedef struct TParameter_tag{
  TParameter_tag();
  ~TParameter_tag();

  int ProcessType;          // -1 = combo, -2 = unknown
  int DisplayType;
  int NumberOfSteps;

  bool Enabled;
  TFileName ParameterName;
  TFileName ParameterDescription;
  TComboLabel *ComboLabels;
  int DefaultValue;
  double DisplayMinValue, DisplayMaxValue, DisplayInitialValue;
  int ParamAddress[MAXSUBPARAM];
  double ParamMinValue[MAXSUBPARAM];
  double ParamMaxValue[MAXSUBPARAM];
  double ParamValue[MAXSUBPARAM];
  TFileName TableFileName[MAXSUBPARAM];
  int *ParamTable[MAXSUBPARAM];         
  int NumberOfSubParams;                // number of sub-params (only <> 1 in groups)
  int ItemInitialized;                  // non-zero if item is successfully initialized
  int PotNumber;                        // number of associated pot

} TParameter;

typedef struct TParameterItem_tag : public TParameter{
  TParameterItem_tag();

  wxTreeItemId id;
  
  // group information
  int InGroup;                          // 0 = alone, 1..5 = param index in a group
  TParameterItem_tag *GroupName;        // in individual param: name of their group
  TParameterItem_tag *SubParams[MAXSUBPARAM]; // in group: references to individual params

  TParameterItem_tag *next;
} TParameterItem;

typedef struct TProjectParameters_tag{
  char ModName[FILENAME_MAX];
  char ModDescr[FILENAME_MAX];
  int AddressOffset;
  TParameterItem *param;

  TProjectParameters_tag *next;
} TProjectParameters;

int AssignDefaultValues(TParameterItem *it);
int LoadProject(const char *ProjectFileName, TProjectParameters **Params, bool LoadTables=false);
int WriteProject(char *ObjectFileName, char *Name, char *map, 
		 TProjectParameters *params);
int DeleteParamList(TParameterItem **it);

#endif
