/////////////////////////////////////////////////////////////////////////////
// Name:        simulate.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.0
// Created:     23/03/03
// Copyright:   (c) 2003 Axoris
/////////////////////////////////////////////////////////////////////////////

#include "simulate.h"
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/valgen.h>

BEGIN_EVENT_TABLE(CSimulate, wxDialog)
  EVT_BUTTON(ID_BTN_INWAV,  CSimulate::OnBrowseInWav)
  EVT_BUTTON(ID_BTN_OUTWAV, CSimulate::OnBrowseOutWav)
  EVT_BUTTON(ID_SIMULATE,   CSimulate::OnSimulate)
END_EVENT_TABLE()

// c'tor & d'tor //
///////////////////

CSimulate::CSimulate(wxWindow* parent, wxWindowID id, char *iAsmFile)
  : wxDialog(parent, id, "Directories", wxDefaultPosition, wxSize(250,170), wxDEFAULT_DIALOG_STYLE){
#ifdef DEBUG
  fprintf(stderr, "DBG: directories: Entering constructor\n");
#endif

  strcpy(AsmFile,iAsmFile);
  int l=strlen(AsmFile);
  AsmFile[l-1]='m';
  AsmFile[l-2]='s';

  SetClientSize(400,250);

  // creates all widgets inside the dialog box
  // Static texts for in & out files
  TxtInWav  = new wxStaticText(this, ID_STATIC, "Input WAV:", 
			       wxPoint(15,20), wxSize(70,20));
  TxtOutWav = new wxStaticText(this, ID_STATIC, "Output WAV:",
			       wxPoint(15,50), wxSize(70,20));

  // edit boxes for in & out files
  EdtInWav  = new wxTextCtrl(this, ID_EDT_INWAV, "", 
			     wxPoint(95, 20), wxSize(220,20),
			     0, wxTextValidator(wxFILTER_ASCII, &InWavName));
  EdtOutWav = new wxTextCtrl(this, ID_EDT_OUTWAV, "", 
			     wxPoint(95, 50), wxSize(220,20),
			     0, wxTextValidator(wxFILTER_ASCII, &OutWavName));

  // 'Browse' buttons for in & out files
  BtnInBrowse = new wxButton(this, ID_BTN_INWAV, "Browse", 
			     wxPoint(330, 20), wxSize(60, 20));
  BtnInBrowse = new wxButton(this, ID_BTN_OUTWAV, "Browse", 
			     wxPoint(330, 50), wxSize(60, 20));

  // actions buttons: simulate, ok & cancel
  BtnSimulate = new wxButton(this, ID_SIMULATE, "Simulate", 
			     wxPoint(300, 110), wxSize(80, 24));

  BtnOk       = new wxButton(this, wxID_OK, "Ok", 
			     wxPoint(310, 170), wxSize(60, 24));
  BtnCancel   = new wxButton(this, wxID_CANCEL, "Cancel", 
			     wxPoint(310, 210), wxSize(60, 24));

  // console for simulation logs
  SimConsole = new wxTextCtrl(this, ID_SIMCONSOLE, "Log window",
			      wxPoint(10, 90), wxSize(270, 160),
			      wxTE_MULTILINE | wxTE_READONLY);

}

CSimulate::~CSimulate(){
#ifdef DEBUG
  fprintf(stderr, "DBG: directories: Entering destructor\n");
#endif
}

// event handling //
////////////////////

// browse for input WAV file
void CSimulate::OnBrowseInWav (wxCommandEvent &e){
  wxString s=wxFileSelector("Choose the input WAV file", "../samples", 
			    "", "", "*.wav", wxOPEN);
  if (!s.IsNull())
    EdtInWav->SetValue(s);
}

// browse for output wav file
void CSimulate::OnBrowseOutWav(wxCommandEvent &e){
  wxString s=wxFileSelector("Choose the output WAV file", "../samples", 
			    "", "", "*.wav", wxSAVE|wxOVERWRITE_PROMPT);
  if (!s.IsNull())
    EdtOutWav->SetValue(s);
}

// Simulate 
void CSimulate::OnSimulate(wxCommandEvent &e){
  // clear the console
  wxString TempStr;
  wxArrayString Output;
  wxArrayString Error;
  int n;
  SimConsole->Clear();
  InWavName  = EdtInWav->GetValue();
  OutWavName = EdtOutWav->GetValue();

  // check if input file is not "" and finishes by ".wav"
  if (strcmp(InWavName, "")==0){
    (*SimConsole) << "No input file specified. Giving up !\n";
    return;
  }
  TempStr = InWavName;
  TempStr = TempStr.MakeLower();
  if ((TempStr.Contains(".wav")==0) 
      && (strcmp(TempStr.AfterLast('.').c_str(), "wav")!=0)){
    (*SimConsole) << "Input file must be a '.wav' file\n";
    return;
  }

  // if out = "", build it from in
  if (strcmp(OutWavName, "")==0){
    OutWavName = InWavName;
    OutWavName.Replace(".wav", "_out.wav");
    (*SimConsole) << "Output file not specified. Using '" << OutWavName << "'.\n";
  }
  TempStr = OutWavName;
  TempStr = TempStr.MakeLower();
  if ((TempStr.Contains(".wav")==0) 
      && (strcmp(TempStr.AfterLast('.').c_str(), "wav")!=0)){
    (*SimConsole) << "Output file must be a '.wav' file\n";
    return;
  }

  // create simulation script
  (*SimConsole) << "Creating Simulation script\n";
  FILE *script=fopen("tmp.scr", "w+");
  fprintf(script, "load %c%s%c\n", '"', AsmFile, '"');
  fprintf(script, "+input file %c%s%c 44100 0 1\n", '"', InWavName.c_str(), '"');
  fprintf(script, "+output file %c%s%c 44100 0 1\n", '"', OutWavName.c_str(), '"');
  fprintf(script, "start blocking\n");
  //fprintf(script, "reset io\n");
  fprintf(script, "exit\n");
  fclose(script);

  // simulate
  (*SimConsole) << "Simulating ... \n";

  int ReturnCode = wxExecute("./alksim tmp.scr exit_if_error", Output, Error);
  if (ReturnCode == 0)
  {
    for (n=0;n<Output.GetCount();n++)
      (*SimConsole) << Output[n] << "\n";
    (*SimConsole) << "Simulation done.\n";
  }
  else if (ReturnCode == -1)
  {
	(*SimConsole) << "\nUnable to find the simulator executable or to execute it\n";
  }
  else
  {
    (*SimConsole) << "Simulation aborted due to errors.\n\n";
    for (n=0;n<Error.GetCount();n++)
      (*SimConsole) << Error[n] << "\n";
  }

  // delete simulation script
//  wxRemoveFile("tmp.scr");

}
