/////////////////////////////////////////////////////////////////////////////
// Name:        theme.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     30/12/03
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __AGALAG_THEME_H__
#define __AGALAG_THEME_H__

#include "Config.h"

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif// Define a new frame type: this is going to be our main frame

#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/listbox.h>
#include <wx/dir.h>
#include <wx/scrolwin.h>
#include <wx/image.h>
#include <wx/checkbox.h>
#include "Config.h"

// theme information structure
typedef struct{
  char Name[128];
  char TexMod[128];
  char TexPin[128];
  int NameColor[3];
  int LinkColor[3];
} TTheme;

int LoadTheme(wxString Name, wxString Path, TTheme *t);
int GetThemeFromUser(CConfig *cfg, TTheme *t);

class CThemePreview : public wxScrolledWindow{
 public:
  CThemePreview(TTheme *i_theme, wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize);
  void OnPaint(wxPaintEvent& event);

  void Redraw();
  void Render(wxDC *dc);

 public: 
  int IsThemeSelected;
  TTheme *theme;
  wxImage mod;
  wxImage pin;

  DECLARE_EVENT_TABLE()
};

class CThemeSelector : public wxDialog{
 public:
  // ctor & dtor
  CThemeSelector(wxWindow* parent, wxWindowID id);
  ~CThemeSelector();

  TTheme NewTheme;

 private:
  bool IsAutoPreview;
  wxButton *BtnOk, *BtnCancel;
  wxStaticText *TxtIntro;
  wxListBox *ThemeList;
  wxCheckBox *AutoPreview;
  wxButton *BtnPreview;
  wxBitmap BmpWindow;
  CThemePreview *Preview;

  int ListThemes();
  int FillTheme(TTheme *t);
  void OnOK(wxCommandEvent &event);
  void OnAutoPreview(wxCommandEvent &event);
  void OnPreview(wxCommandEvent &event);
  void OnThemeSelected(wxCommandEvent &event);

  DECLARE_EVENT_TABLE()
};

#endif
