//--------------------------------------------------------------------//
//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: replace.cpp                                                 //
//  Description: functions dealing with strings tokens modifications  //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//

#include "replace.h"
#include "tokenize.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "messages.h"

/////////////////////////////////////////////////////////////
// Compare strings without taking case into account
// This returns the same result as strcmp except that there
// is no dinstinction between upper case and lower case
/////////////////////////////////////////////////////////////

int CompareStrings(char *s1, char *s2){
  int i, l1, l2;
  char s1_low[255];
  char s2_low[255];

  l1 = strlen(s1);
  l2 = strlen(s2);

  for (i=0;i<l1;i++) s1_low[i]=tolower(s1[i]);
  for (i=0;i<l2;i++) s2_low[i]=tolower(s2[i]);

  s1_low[l1]=s2_low[l2]='\0';

  return strcmp(s1_low, s2_low);
}

/////////////////////////////////////////////////////////////
// CReplacementList implementation
/////////////////////////////////////////////////////////////

// ctor and dtor
CReplacementList::CReplacementList(){
  List=NULL;
  Nb=0;
}

CReplacementList::~CReplacementList(){
  Reset();
}

// Add an element to the list
int CReplacementList::Add(char *i_pattern, char *i_newval){
  TReplacementList *Tmp;

  // check if the new pattern is not already present
  Tmp=List;
  while (Tmp!=NULL){
    if (CompareStrings(Tmp->Replacement->Pattern, i_pattern)==0){
      // the pattern exists ... returns !
      WARNING("There are multiply defined patterns\nPlease submit a full bug report to Gregor.Soyez@swing.be\n");
      return REPLACE_EXISTS;
    }
    Tmp=(TReplacementList*) Tmp->Next;
  };

  // record new pattern
  Tmp=new TReplacementList;
  Tmp->Next=(void*) List;
  List=Tmp;
  List->Replacement = new TStrReplace;
  List->Replacement->Pattern=new char[strlen(i_pattern)+1];
  strcpy(List->Replacement->Pattern, i_pattern);
  List->Replacement->NewVal=new char[strlen(i_newval)+1];
  strcpy(List->Replacement->NewVal, i_newval);

  Nb++;

  return REPLACE_OK;
}

// Modify New value of the given patternn
// The last parameter tells what to do if the pattern not exists 
// it must be REPLACE_IGNORE or REPLACE_ADD
// The function returns REPLACE_EXISTS or REPLACE_NOT_EXISTS.
int CReplacementList::Modify(char *i_pattern, char *i_newval, int Mode){
  TReplacementList *Tmp;

  // browse list and search for the pattern
  Tmp=List;
  while (Tmp!=NULL){
    if (CompareStrings(Tmp->Replacement->Pattern, i_pattern)==0){
      // the pattern exists ...
      delete[] Tmp->Replacement->NewVal;
      List->Replacement->NewVal=new char[strlen(i_newval)+1];
      strcpy(List->Replacement->NewVal, i_newval);
      return REPLACE_EXISTS;
    }
    Tmp=(TReplacementList*) Tmp->Next;
  };

  // the requested pattern is not defined. Add it if mode tells so.
  if (Mode==REPLACE_ADD){
    Add(i_pattern, i_newval);
  }

  return REPLACE_NOT_EXISTS;
}

// remove an element from the list
int CReplacementList::Remove(char *i_pattern){
  TReplacementList *Tmp, *BeforeTmp;

  // browse list and search for the pattern
  Tmp=List;
  BeforeTmp=NULL;
  while (Tmp!=NULL){
    if (CompareStrings(Tmp->Replacement->Pattern, i_pattern)==0){
      // the pattern exists ...
      delete[] Tmp->Replacement->Pattern;
      delete[] Tmp->Replacement->NewVal;
      delete Tmp->Replacement;
      if (BeforeTmp==NULL){
	// the requested element was the first one !)
	List=(TReplacementList*) Tmp->Next;
      } else {
	// we have to remove an element in the middle of the list
	BeforeTmp->Next=Tmp->Next;
      }
      delete Tmp;
      Nb--;
      return REPLACE_OK;
    }
    BeforeTmp=Tmp;
    Tmp=(TReplacementList*) Tmp->Next;
  };

  return REPLACE_NOT_EXISTS;
}

// reset the list
int CReplacementList::Reset(){
  TReplacementList *Tmp;
  while (List!=NULL){
    Tmp=(TReplacementList*) List->Next;
    delete[] List->Replacement->Pattern;
    delete[] List->Replacement->NewVal;
    delete List->Replacement;
    delete List;
    List=Tmp;
  };
  Nb=0;
  
  return REPLACE_OK;
}

// returns the modified string (arg1) through arg2.
// output string is supposed to be allocated
int CReplacementList::Perform(char *istr, char *rstr){
  int n, i, *done;
  char **tokens;
  TReplacementList *Tmp;

  // retreive all the tokens of the initial string
  n=StrTokenize(istr, &tokens);

  if (n==0){
    strcpy(rstr, istr);
    return REPLACE_OK;
  }

  // create a table with all replaced tokens
  done=new int[n];
  for (i=0;i<n;i++) done[i]=0;

  // browse all replacement patterns
  Tmp=List;
  while (Tmp!=NULL){
    // compare pattern with all tokens (we do not care about case !!)
    for (i=0;i<n;i++){
      if ((CompareStrings(tokens[i], Tmp->Replacement->Pattern)==0) && (done[i]==0)){
	// replace the token by its new value.
	delete[] tokens[i];
	tokens[i]=new char[strlen(Tmp->Replacement->NewVal)+1];
	strcpy(tokens[i], Tmp->Replacement->NewVal);
	done[i]=1;
      }
    }
    Tmp=(TReplacementList*) Tmp->Next;
  };
  
  // construct new string from new tokens
  // NOTE: THE DELIMITATION CARACTERS SHOULD BE AT THE SAME PLACE 
  // THAN THOSE OF THE INITIAL STRING !
  // THIS IS NOT IMPLEMENTED IN THE CURRENT VERSION !!
  strcpy(rstr, tokens[0]);
  for (i=1;i<n;i++){
    strcat(rstr, " ");
    strcat(rstr, tokens[i]);
  }

  // free memory
  for (i=0;i<n;i++)
    delete[] tokens[i];
  delete[] tokens;
  delete[] done;

  return REPLACE_OK;
}
