//--------------------------------------------------------------------//
//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: replace.h                                                   //
//  Description: functions dealing with strings tokens modifications  //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//

#ifndef __PROJ_AL_REPLACE_H__
#define __PROJ_AL_REPLACE_H__

#include "ProjAL.h"

#define REPLACE_OK          0
#define REPLACE_EXISTS      1
#define REPLACE_NOT_EXISTS  2

#define REPLACE_IGNORE      0
#define REPLACE_ADD         1

/*
** During the parsing, we will have various replacemenets to
** perform such as OUT1 by DIR0 or FRQ by MeDisto1FRQ
** We thus create a struct for that kind of replacements
** as well as a dynamic list of these objects
** ToOffset tells wether or not that replacement has
** to take Memory offset into account
*/
typedef struct{
  char *Pattern;
  char *NewVal;
} TStrReplace;

typedef struct{
  TStrReplace *Replacement;
  void *Next;
} TReplacementList;

class CReplacementList{
 public:
  // ctor and dtor
  CReplacementList();
  ~CReplacementList();

  // Add an element to the list
  // Returns REPLACE_OK or REPLACE_EXISTS
  int Add(char*, char*);

  // Modify New value of the given patternn
  // The last parameter tells what to do if the pattern not exists 
  // it must be REPLACE_IGNORE or REPLACE_ADD
  // The function returns REPLACE_EXISTS or REPLACE_NOT_EXISTS.
  int Modify(char*, char*, int);

  // remove an element from the list
  // Returns REPLACE_OK or REPLACE_NOT_EXISTS
  int Remove(char*);

  // reset the list
  // Returns REPLACE_OK
  int Reset();

  // returns the modified string (arg1) through arg2.
  // output string is supposed to be allocated
  // The third parameter tells what to add for memory offsets
  int Perform(char*, char*);

 private:
  int Nb;
  TReplacementList *List;
};

int CompareStrings(char *s1, char *s2);

#endif
