#include "../../common/1K_types.h"
#define EXTERN_PREASM_DEFINED
#include "PreAsm.h"
#include <string.h>
#include <stdio.h>

#define NB_CST    2048
#define NB_LABEL  256

// Preassembler variables.
PAsmCst *AL3101PreAsmCstTable;      // ABS and EQU tables
PAsmLabel *AL3101PreAsmLabelTable;  // labels table
int AL3101PreAsmNbCst;              // Number of predefined constants
int AL3101PreAsmNbLabel;            // number of labels
int AL3101PreAsmProgramCounter;     // when assembling step, this is the PC.
                                    // it is used for labels 

// Check if the given string is already present in the CstTable
// return -1 or the index of the already existing constant
int PreAsmCheckCstExistence(char *Cst){
  int i;
  i=0;
  while (i<AL3101PreAsmNbCst){
    if (strcmp(AL3101PreAsmCstTable[i].Name, Cst)==0) return i;
    i++;
  }

  return -1;
}

// Check if the given string is already present in the LabelTable
// return -1 or the index of the already existing constant
int PreAsmCheckLabelExistence(char *Label){
  int i;
  i=0;
  while (i<AL3101PreAsmNbLabel){
    if (strcmp(AL3101PreAsmLabelTable[i].Name, Label)==0) return i;
    i++;
  }

  return -1;
}

// Check if the given string is already present in any table
// return -1 if it doesn't exists, 0 if it exists
int PreAsmCheckExistence(char *Name){
  if (PreAsmCheckCstExistence(Name)!=-1)
    return 0;
  if (PreAsmCheckLabelExistence(Name)!=-1)
    return 0;

  return -1;
}

// Add a pre-assembler-defined constant to the existing list
// This function returns PREASM_OK if successful and PREASM_MAX_CST if
// pre-assembler capacity exceeded and PREASM_EXISTS if the constant already exists.
int PreAsmAddCst(char *Name, char *Value, int Type){
  // check capacity
  if (AL3101PreAsmNbCst==NB_CST)
    return PREASM_MAX_CST;

  // check if already exists IN BOTH labels and cst tables !
  if (PreAsmCheckExistence(Name)!=-1)
    return PREASM_EXISTS;

  AL3101PreAsmCstTable[AL3101PreAsmNbCst].Name=new char[strlen(Name)+1];
  AL3101PreAsmCstTable[AL3101PreAsmNbCst].StrValue=new char[strlen(Value)+1];

  strcpy(AL3101PreAsmCstTable[AL3101PreAsmNbCst].Name,Name);
  strcpy(AL3101PreAsmCstTable[AL3101PreAsmNbCst].StrValue,Value);
  AL3101PreAsmCstTable[AL3101PreAsmNbCst].Type = Type;

  AL3101PreAsmNbCst++;
  return PREASM_OK;
}

// Add a pre-assembler-defined label to the existing list
// This function returns PREASM_OK if successful and PREASM_MAX_LABEL if
// pre-assembler capacity exceeded and PREASM_EXISTS if the label already exists
// as a label or as a constant
int PreAsmAddLabel(char *Name, int CmdIndex){
  int len = (int) strlen(Name);
  char Tmp[255];

  // exclude ':' at the end of Name
  strcpy(Tmp, Name);
  Tmp[strlen(Name)-1]='\0';

  // check capacity
  if (AL3101PreAsmNbLabel==NB_LABEL)
    return PREASM_MAX_CST;

  // check if already exists IN BOTH labels and cst tables !
  if (PreAsmCheckExistence(Tmp)!=-1)
    return PREASM_EXISTS;

  AL3101PreAsmLabelTable[AL3101PreAsmNbLabel].Name=new char[len+1];

  strcpy(AL3101PreAsmLabelTable[AL3101PreAsmNbLabel].Name,Tmp);
  AL3101PreAsmLabelTable[AL3101PreAsmNbLabel].CmdIndex = CmdIndex;

  AL3101PreAsmNbLabel++;
  return PREASM_OK;
}

// Initialize the preassembler
// This add the preasm constants always defined by default
// It returns PREASM_OK when successful.
int InitPreAsm(){
  AL3101PreAsmCstTable = new PAsmCst[NB_CST];
  AL3101PreAsmLabelTable = new PAsmLabel[NB_LABEL];

  AL3101PreAsmNbCst = 0;
  AL3101PreAsmNbLabel = 0;

  AL3101PreAsmProgramCounter = 0;

  PreAsmAddCst("IN1", "0X410", 1);  PreAsmAddCst("OUT1", "0X410", 1);
  PreAsmAddCst("IN2", "0X411", 1);  PreAsmAddCst("OUT2", "0X411", 1);
  PreAsmAddCst("IN3", "0X412", 1);  PreAsmAddCst("OUT3", "0X412", 1);
  PreAsmAddCst("IN4", "0X413", 1);  PreAsmAddCst("OUT4", "0X413", 1);
  PreAsmAddCst("IN5", "0X414", 1);  PreAsmAddCst("OUT5", "0X414", 1);
  PreAsmAddCst("IN6", "0X415", 1);  PreAsmAddCst("OUT6", "0X415", 1);
  PreAsmAddCst("IN7", "0X416", 1);  PreAsmAddCst("OUT7", "0X416", 1);
  PreAsmAddCst("IN8", "0X417", 1);  PreAsmAddCst("OUT8", "0X417", 1);

  PreAsmAddCst("DIR0", "0X400", 1);  PreAsmAddCst("DIROUT0", "0X420", 1);
  PreAsmAddCst("DIR1", "0X401", 1);  PreAsmAddCst("DIROUT1", "0X421", 1);
  PreAsmAddCst("DIR2", "0X402", 1);  PreAsmAddCst("DIROUT2", "0X422", 1);
  PreAsmAddCst("DIR3", "0X403", 1);  PreAsmAddCst("DIROUT3", "0X423", 1);
  PreAsmAddCst("DIR4", "0X404", 1);  PreAsmAddCst("DIROUT4", "0X424", 1);
  PreAsmAddCst("DIR5", "0X405", 1);  PreAsmAddCst("DIROUT4", "0X425", 1);
  PreAsmAddCst("DIR6", "0X406", 1);  PreAsmAddCst("DIROUT6", "0X426", 1);
  PreAsmAddCst("DIR7", "0X407", 1);  PreAsmAddCst("DIROUT7", "0X427", 1);
  PreAsmAddCst("DIR8", "0X408", 1);  PreAsmAddCst("DIROUT8", "0X428", 1);
  PreAsmAddCst("DIR9", "0X409", 1);  PreAsmAddCst("DIROUT9", "0X429", 1);
  PreAsmAddCst("DIRA", "0X40A", 1);  PreAsmAddCst("DIROUTA", "0X42A", 1);
  PreAsmAddCst("DIRB", "0X40B", 1);  PreAsmAddCst("DIROUTB", "0X42B", 1);
  PreAsmAddCst("DIRC", "0X40C", 1);  PreAsmAddCst("DIROUTC", "0X42C", 1);
  PreAsmAddCst("DIRD", "0X40D", 1);  PreAsmAddCst("DIROUTD", "0X42D", 1);
  PreAsmAddCst("DIRE", "0X40E", 1);  PreAsmAddCst("DIROUTE", "0X42E", 1);
  PreAsmAddCst("DIRF", "0X40F", 1);  PreAsmAddCst("DIROUTF", "0X42F", 1);
    
  PreAsmAddCst("LOG16","0X418", 1);  PreAsmAddCst("DIRIN","0X419", 1);

  return PREASM_OK;
}


// Preassemble a command
// This should be called by the main assembler loop before treating
// the command as a assembler one.
// The function returns PREASM_OK if it is a valid preasm command, 
// PREASM_SYNTAX if it is a preasm command with wrong syntax, PREASM_FORMAT
// if the name of the preasm-defined string is not valid, PREASM_EXISTS if
// the preassembler-defined string already exists and PREASM_NOTHING
// if the command doesn't correspond to any preassembler command.
// Note : CmdIndex is the index of the first line after the label.
int PreAssemble(int nb_tok, char *Tokens[], int CmdIndex){
  // initial vaules are given to prevent from compilation warnings 
  int i=0;
  int PreAsmToken=CST_EQU; // token containing defined string

  // check EQU and ABS command
  if (((nb_tok!=1) && (strcmp(Tokens[1], "EQU")==0)) || (strcmp(Tokens[0], "ABS")==0)){
    if (strcmp(Tokens[0], "ABS")!=0) 
      PreAsmToken=0;  // <String> EQU <Value>
    else
      PreAsmToken=1;  // ABS <String> <Value>

    // check if 3 tokens
    if (nb_tok!=3)
      return PREASM_SYNTAX;

    // Note : At this point, we choose to treat the command ABS EQU <val>
    // In the current code, we tried to stay as close as possible from the 
    // Alesis assembler : it is understood as only producing a warning without
    // updating any table. This results in error when trying to access
    // ABS or EQU in commands. The command ABS EQU <val> is thus accepted by 
    // the preassembler and simply returns PREASM_OK.
    if ((strcmp(Tokens[0], "ABS")==0) && (strcmp(Tokens[1], "EQU")==0)){
      fprintf(stderr, "Preassembler Warning: command ABS EQU <value> has no effect\n");
      return PREASM_OK;
    }

    // check syntax for the Name.
    // This should be an alphanumeric string starting with a letter
    if ((Tokens[PreAsmToken][0]<'A') || (Tokens[PreAsmToken][0]>'Z'))
      return PREASM_FORMAT;
    for (i=1;i<(int) strlen(Tokens[PreAsmToken]);i++){
      if (((Tokens[PreAsmToken][i]<'0') || 
	   ((Tokens[PreAsmToken][i]>'9') && (Tokens[PreAsmToken][i]<'A')) ||
	  (Tokens[PreAsmToken][i]>'Z'))
	  && (Tokens[PreAsmToken][i]!='_'))
	return PREASM_FORMAT;
    }

    // we don't worry about the constant syntax. 
    // In case of problems, it will be seen during assembling
    // Record the new statement in the Constant table.
    // This utomatically checks if PreAsm capacity is not exceeded and if the new
    // constant deosn't already exists.
    return PreAsmAddCst(Tokens[PreAsmToken], Tokens[2],
			((strcmp(Tokens[1],"EQU")==0) ? CST_EQU : CST_ABS));
  }



  // Now, let's deal with labels
  if ((nb_tok==1) && (Tokens[0][strlen(Tokens[0])-1]==':')){
    // check syntax for the Name.
    // This should be an alphanumeric string starting with a letter
    if ((Tokens[0][0]<'A') || (Tokens[0][0]>'Z'))
      return PREASM_FORMAT;
    for (i=1;i<(int) strlen(Tokens[0])-1;i++){
      if (((Tokens[0][i]<'0') || 
	   ((Tokens[0][i]>'9') && (Tokens[0][i]<'A')) ||
	   (Tokens[0][i]>'Z'))
	  && (Tokens[0][i]!='_'))
	return PREASM_FORMAT;
    }
    
    // we don't worry about the constant syntax. 
    // In case of problems, it will be seen during assembling
    // Record the new statement in the Constant table.
    // This utomatically checks if PreAsm capacity is not exceeded and if the new
    // constant deosn't already exists.
    return PreAsmAddLabel(Tokens[0], CmdIndex);
  }
  
  return PREASM_NOTHING;
}

// update the Preasm PC (increment it by 1)
// returns PREASM_OK
int PreAsmStepPC(){
  AL3101PreAsmProgramCounter+=1;

  return PREASM_OK;
}

// given a string, check if it correspond to a given predefined constant. 
// If it is the case, put the sting corresponding to the searched value 
// into strCst and return 0. Otherwise, return 1.
// IsAddr is 1 if wo need an integer value (i.e an address). 
// Function returns PREASM_OK if value returned successfull and PREASM_NOTHING
// if requested constant doesn't exists.
int PreAsmGetConstant(char *str, char *strCst, int Type){
  int i;

  // Three Types are allowed: CST_ABS, CST_EQU and CST_LABEL
  // All types are exclusive except for the CST_LABEL case for which we also
  // have to check the CST_EQU values.

  // start by treating LABEL
  if (Type==CST_LABEL){
    for (i=0;i<AL3101PreAsmNbLabel;i++){
      if (strcmp(AL3101PreAsmLabelTable[i].Name, str)==0){
	// Here is the point where we need to know the actual position 
	// in the asm code !!!
	sprintf(strCst, "%d", AL3101PreAsmLabelTable[i].CmdIndex-AL3101PreAsmProgramCounter-1);
	return PREASM_OK;
      }
    }
    // search unsuccessful ... go and see in the CST Table for CST_EQU
    Type=CST_EQU;
  }

  // treat EQU and ABS
  for (i=0;i<AL3101PreAsmNbCst;i++){
    if ((strcmp(AL3101PreAsmCstTable[i].Name, str)==0) && (AL3101PreAsmCstTable[i].Type==Type)){
      sprintf(strCst, "%s", AL3101PreAsmCstTable[i].StrValue);
      return PREASM_OK;
    }
  }

  return PREASM_NOTHING;
}

// Close Preasm
// This function basically release all allocated memory
// returns PREASM_OK
int PreAsmClose(){
  int i;

  for (i=0;i<AL3101PreAsmNbCst;i++){
    delete AL3101PreAsmCstTable[i].Name;
    delete AL3101PreAsmCstTable[i].StrValue;
  }
  delete[] AL3101PreAsmCstTable;

  for (i=0;i<AL3101PreAsmNbLabel;i++){
    delete AL3101PreAsmLabelTable[i].Name;
  }
  delete[] AL3101PreAsmLabelTable;

  return PREASM_OK;
}

