#include <stdio.h>
#include <string.h>
#include "AL3101DisasmFunc.h"
#include "AL3101AsmDisasmUtils.h"

// conditions
char* DisasmConditionTable[] = {
  "!Z",
  "Z",
  "!V",
  "V",
  "N",
  "!N",
  "VN",
  "V!N",
  "!VN",
  "!V!N",
  "!V!N!Z",
  "!N!Z",
  "N+Z",
  "!V(N+Z)",
  "",
  "VZ"
};

// DISASSEMBLING FUNCTIONS
// all the assembling functions have the same structure :
//   they take the 2 higher bits, the 32 lower bits, and a pointer to 
//   a string in entry. The returned value is 0 if sucess or the -index of
//   the failing parameter. Disassembled parameters are returned
//   pointer passed as parameter. If the mnemonic requires an extrabit, the
//   extrabit is returned.
// WARNING : all constants with floating point format (c & d fields) are 
//           stored on 28 bits (S3.24)
 
// normal mnemonics (no extrabit, no condition)
int disasm_c18_x_m(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<26) + ((lparam>>12)<<6);
  out->m = (lparam%0x800);

  sprintf(sout, "0x%x 0x%x", ((out->c)>>6), out->m);
  ExtendTo32Bits(&(out->c));
  return 0;
}

int disasm_x_m(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->m = (lparam%0x800);

  sprintf(sout, "0x%x", out->m);
  return 0;
}

int disasm_c24_x(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<26) + (lparam>>6);

  sprintf(sout, "0x%x", out->c);
  ExtendTo32Bits(&(out->c));
  return 0;
}

int disasm_c18_x(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<26) + ((lparam>>12)<<6);

  sprintf(sout, "0x%x", ((out->c)>>6));
  ExtendTo32Bits(&(out->c));
  return 0;
}

int disasm_c18_d8(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<26) + ((lparam>>12)<<6);
  out->d = ((lparam%0x1000) << 16);

  sprintf(sout, "0x%x 0x%x", ((out->c)>>6), ((out->d)>>16));
  ExtendTo32Bits(&(out->c));
  ExtendTo32Bits(&(out->d));
  return 0;
}

int disasm_c8_d18(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<26) + ((lparam>>22)<<16);
  out->d = ((lparam%0x400000)<<6);

  sprintf(sout, "0x%x 0x%x", ((out->c)>>16), ((out->d)>>6));
  ExtendTo32Bits(&(out->c));
  ExtendTo32Bits(&(out->d));
  return 0;
}

int disasm_x(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  sprintf(sout, "");
  return 0;
}

// mnemonics with condition
int disasm_c24_x_p(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<26) + (lparam>>6);
  out->m = (lparam%0x10);

  sprintf(sout, "%s 0x%x", DisasmConditionTable[out->m], out->c);
  ExtendTo32Bits(&(out->c));
  return 0;
}

int disasm_c8_x_p(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  out->c = (wparam<<6) + (lparam>>26);  // no shift
  out->m = (lparam%0x10);

  sprintf(sout, "%s 0x%x", DisasmConditionTable[out->m], out->c);
  return 0;
}

// mnemonics with extra bit
int disasm_c10_x_b_m(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  int eb = ((lparam>>11)%0x2);
  out->c  = (wparam<<8) + (lparam>>24);  // no shift !!!
  out->m  = (lparam%0x800);

  sprintf(sout, "0x%x 0x%x", out->c, out->m);
  return eb;
}

int disasm_c24_b_x(UINT32 wparam, UINT32 lparam, AL3101_Disasm_struct *out, char *sout){
  int eb = ((lparam>>5)%0x2);
  
  if (eb){
    out->c = (wparam<<26) + (lparam>>6);
    sprintf(sout, "0x%x", out->c);
    ExtendTo32Bits(&(out->c));
  } else
    sprintf(sout, "");

  return eb;
}
