#include "../../common/1K_types.h"
#include "AL3101Struct.h"
#include <math.h>			// for the math functions used in main

inline INT32 mac1(INT32 m28, INT32 m24, INT32 a28, INT32 rounding, INT32 *n, INT32 *z, INT32 *v)
{
	INT64 temp64;
	INT32 temp32;

	// The multiplication of a S3.24 value by a S3.18 value gives a SS6.42 value
	// However two assumptions are made:
	//	1) Here the sign bit is stored in the sign bit of the INT64 value. As a consequence,
	//		the operation is the multiplication of a signed 3.24 by a signed 3.18 values
	//		leading to a signed 6.42 value
	//	2) Moreover, the values are aligned for ease of programming meaning that
	//		the operation is the multiplication of a signed 3.24 by a signed 3.24 values
	//		(the second one has only 18 representative fractional bits)
	//		leading to a signed 6.48 value
	temp64 = (INT64)m28 * (INT64)m24;
	// The result of this multiplication has to be brought back to a signed 3.24 value
	// First, test if rounding has to be made on the result
	//  if a truncation is required, then nothing is done at that point
	//  the truncation will be done later on 
	if (rounding == 1)	// rounding has to be done
	{
		temp32 = (INT32)(temp64 & (INT64)0x0003FFFF);
		if (temp32 > 0x0001FFFF) temp64 = temp64 + (INT64)0x00040000;
	}
	// Secondly, shrink the 6.48 value to a 6.24 one
	// Eliminate first the 24 lowest bits generated by the multiplication
	temp32 = (INT32)(temp64 >> 24);
	
	// Then add the addend to the result
	temp32 = temp32 + a28;

	// Finally, remove the 3 upper bits of the 6.24 value
	// if a saturation has to be made, it must be done here
	// Setting of the overflow bit  will be done here
	*v = 0;
	if (temp32 > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		if (temp32 > (INT32)0x07FFFFFF)
		{
			*v = 1;
			/*// set the upper bits to 0 by ANDing the lower bits with 1
			temp32 = temp32 & 0x07FFFFFF;
			*/
			temp32 = (INT32)0x07FFFFFF;
		}
		*n = 0;
		*z = 0;
	}
	else if (temp32 < (INT32)0)
	{				// if result is negative
		if (temp32 < (INT32)0xF8000000)
		{
			*v = 1;
			/*// set the upper bits to 1 by ORing the upper bits with 1 
			temp32 = temp32 | 0xF8000000;*/
			temp32 = (INT32)0xF8000000;
		}
		*n = 1;
		*z = 0;
	}
	else
	{
		*n = 0;
		*z = 1;
	}

	return temp32;
}


inline INT32 mult1(INT32 m28, INT32 m24, INT32 rounding, INT32 *n, INT32 *z, INT32 *v)
{
	INT64 temp64;
	INT32 temp32;

	// The multiplication of a S3.24 value by a S3.18 value gives a SS6.42 value
	// However two assumptions are made:
	//	1) Here the sign bit is stored in the sign bit of the INT64 value. As a consequence,
	//		the operation is the multiplication of a signed 3.24 by a signed 3.18 values
	//		leading to a signed 6.42 value
	//	2) Moreover, the values are aligned for ease of programming meaning that
	//		the operation is the multiplication of a signed 3.24 by a signed 3.24 values
	//		(the second one has only 18 representative fractional bits)
	//		leading to a signed 6.48 value
	temp64 = (INT64)m28 * (INT64)m24;
	// The result of this multiplication has to be brought back to a signed 3.24 value
	// First, test if rounding has to be made on the result
	//  if a truncation is required, then nothing is done at that point
	//  the truncation will be done later on 
	if (rounding == 1)	// rounding has to be done
	{
		temp32 = (INT32)(temp64 & (INT64)0x0003FFFF);
		if (temp32 > (INT32)0x0001FFFF) temp64 = temp64 + (INT64)0x00040000;
	}
	// Secondly, shrink the 6.48 value to a 6.24 one
	// Eliminate first the 24 lowest bits generated by the multiplication
	temp32 = (INT32)(temp64 >> 24);
	
	// Finally, remove the 3 upper bits of the 6.24 value
	// if a saturation has to be made, it must be done here
	// Setting of the overflow bit  will be done here
	*v = 0;
	if (temp32 > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		if (temp32 > (INT32)0x07FFFFFF)
		{
			*v = 1;
			/*// set the upper bits to 0 by ANDing the lower bits with 1
			temp32 = temp32 & 0x07FFFFFF;*/
			temp32 = (INT32)0x07FFFFFF;
		}
		*n = 0;
		*z = 0;
	}
	else if (temp32 < (INT32)0)
	{				// if result is negative
		if (temp32 < (INT32)0xF8000000)
		{
			*v = 1;
			/*// set the upper bits to 1 by ORing the upper bits with 1 
			temp32 = temp32 | 0xF8000000;*/
			temp32 = (INT32)0xF8000000;
		}
		*n = 1;
		*z = 0;
	}
	else
	{
		*n = 0;
		*z = 1;
	}

	return temp32;
}


inline INT32 add1(INT32 m28, INT32 a28, INT32 *n, INT32 *z, INT32 *v)
{
	INT32 temp32;

	// Then add the addend to the result
	temp32 = m28 + a28;

	// Finally, remove the 3 upper bits of the 6.24 value
	// if a saturation has to be made, it must be done here
	// Setting of the overflow bit  will be done here
	*v = 0;
	if (temp32 > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		if (temp32 > (INT32)0x07FFFFFF)
		{
			*v = 1;
			/*// set the upper bits to 0 by ANDing the lower bits with 1
			temp32 = temp32 & 0x07FFFFFF;*/
			temp32 = (INT32)0x07FFFFFF;
		}
		*n = 0;
		*z = 0;
	}
	else if (temp32 < (INT32)0)
	{				// if result is negative
		if (temp32 < (INT32)0xF8000000)
		{
			*v = 1;
			/*// set the upper bits to 1 by ORing the upper bits with 1 
			temp32 = temp32 | 0xF8000000;*/
			temp32 = (INT32)0xF8000000;
		}
		*n = 1;
		*z = 0;
	}
	else
	{
		*n = 0;
		*z = 1;
	}

	return temp32;
}


inline INT32 exp1(INT32 m28, INT32 *n, INT32 *z, INT32 *v)
{
	INT32 temp32;
	FLOAT32 tempfl;

	// convert the fixed-point value to a floating-point value
	tempfl = (FLOAT32)m28 / (FLOAT32)0x1000000;
	// take the exp of the number
	tempfl = (FLOAT32)pow(16, tempfl);
	// convert back the floating-point value to a fixed-point value
	temp32 = (INT32)(tempfl * (FLOAT32)0x1000000);

	// Finally, remove the upper bits of the 7.24 value
	// if a saturation has to be made, it must be done here
	// Setting of the overflow bit  will be done here
	*v = 0;
	if (temp32 > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		if (temp32 > (INT32)0x07FFFFFF) *v = 1;
		// set the upper bits to 0 by ANDing the lower bits with 1
		temp32 = temp32 & (INT32)0x07FFFFFF;
		*n = 0;
		*z = 0;
	}
	else if (temp32 < (INT32)0)
	{				// if result is negative
		if (temp32 < (INT32)0xF8000000) *v = 1;
		// set the upper bits to 1 by ORing the upper bits with 1 
		temp32 = temp32 | (INT32)0xF8000000;
		*n = 1;
		*z = 0;
	}
	else
	{
		*n = 0;
		*z = 1;
	}

	return temp32;
}


inline INT32 log1(INT32 m28, INT32 *n, INT32 *z, INT32 *v)
{
	INT32 temp32;
	FLOAT32 tempfl;

	// convert the fixed-point value to a floating-point value
	tempfl = (FLOAT32)m28 / (FLOAT32)0x1000000;
	// take the exp of the number
	tempfl = (FLOAT32)log10(tempfl) / (FLOAT32)log10(16);
	// convert back the floating-point value to a fixed-point value
	temp32 = (INT32)(tempfl * (FLOAT32)0x1000000);

	// Finally, remove the upper bits of the 7.24 value
	// if a saturation has to be made, it must be done here
	// Setting of the overflow bit  will be done here
	*v = 0;
	if (temp32 > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		if (temp32 > (INT32)0x07FFFFFF) *v = 1;
		// set the upper bits to 0 by ANDing the lower bits with 1
		temp32 = temp32 & (INT32)0x07FFFFFF;
		*n = 0;
		*z = 0;
	}
	else if (temp32 < (INT32)0)
	{				// if result is negative
		if (temp32 < (INT32)0xF8000000) *v = 1;
		// set the upper bits to 1 by ORing the upper bits with 1 
		temp32 = temp32 | (INT32)0xF8000000;
		*n = 1;
		*z = 0;
	}
	else
	{
		*n = 0;
		*z = 1;
	}

	return temp32;
}


void _1AC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = add1(AL3101->A, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _L1AC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->Data[AL3101_Disasm->m];		// memory transfer

	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = add1(AL3101->A, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _S1AC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = add1(AL3101->A, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SX1AC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->B = AL3101->A;							// register transfer

	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = add1(AL3101->A, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _X1AC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->A;							// register transfer

	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = add1(AL3101->A, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _1BC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->B;			// update the U register with the last value used

	AL3101->A = add1(AL3101->B, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _S1BC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->U = AL3101->B;			// update the U register with the last value used

	AL3101->A = add1(AL3101->B, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _1MC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->Data[AL3101_Disasm->m];		// update the U register with the last value used

	AL3101->A = add1(AL3101->U, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _L1MC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->Data[AL3101_Disasm->m];		// memory transfer

	AL3101->U = AL3101->Data[AL3101_Disasm->m];		// update the U register with the last value used

	AL3101->A = add1(AL3101->U, AL3101_Disasm->c, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _AAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LAAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->B = AL3101->Data[AL3101_Disasm->m];		// memory transfer

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SAAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SXAAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer
	AL3101->B = AL3101->A;							// register transfer

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->A;			// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _AMB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, m24, AL3101->B, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _AMC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LAMC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->B = AL3101->Data[AL3101_Disasm->m];		// memory transfer

	m24 = AL3101->A & 0xFFFFFFF0;
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SAUC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	m24 = AL3101->A & 0xFFFFFFF0;
	// The U register is up-to-date and will be used as this

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _BAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	m24 = AL3101->B & 0xFFFFFFF0;
	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SBAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	m24 = AL3101->B & 0xFFFFFFF0;
	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _BBC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	m24 = AL3101->B & 0xFFFFFFF0;
	AL3101->U = AL3101->B;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->B, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SBBC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	m24 = AL3101->B & 0xFFFFFFF0;
	AL3101->U = AL3101->B;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->B, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _BMC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	m24 = AL3101->B & 0xFFFFFFF0;
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SBUA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	m24 = AL3101->B & 0xFFFFFFF0;
	// The U register is up-to-date and will be used as this

	AL3101->A = mac1(AL3101->U, m24, AL3101->A, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SBUC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	m24 = AL3101->B & 0xFFFFFFF0;
	// The U register is up-to-date and will be used as this

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LCA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->Data[AL3101_Disasm->m];		// memory transfer

	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mult1(AL3101->A, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mult1(AL3101->A, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SXCA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer
	AL3101->B = AL3101->A;							// register transfer

	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mult1(AL3101->A, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CAB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, AL3101_Disasm->c, AL3101->B, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCAB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, AL3101_Disasm->c, AL3101->B, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CAD(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, AL3101_Disasm->c, AL3101_Disasm->d, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CAM(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, AL3101_Disasm->c, AL3101->Data[AL3101_Disasm->m], AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->B;							// update the U register with the last value used

	AL3101->A = mult1(AL3101->B, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->U = AL3101->B;							// update the U register with the last value used

	AL3101->A = mult1(AL3101->B, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCBA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->U = AL3101->B;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->B, AL3101_Disasm->c, AL3101->A, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CBM(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->B;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->B, AL3101_Disasm->c, AL3101->Data[AL3101_Disasm->m], AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CM(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mult1(AL3101->U, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LCM(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->Data[AL3101_Disasm->m];			// memory transfer

	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mult1(AL3101->U, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}

void _XCM(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->A;								// register transfer

	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mult1(AL3101->U, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CMA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, AL3101_Disasm->c, AL3101->A, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LCMA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->Data[AL3101_Disasm->m];			// memory transfer

	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, AL3101_Disasm->c, AL3101->A, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _XCMA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->B = AL3101->A;								// register transfer

	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, AL3101_Disasm->c, AL3101->A, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _CMB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used

	AL3101->A = mac1(AL3101->U, AL3101_Disasm->c, AL3101->B, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCU(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	// The U register is up-to-date and will be used as this

	AL3101->A = mult1(AL3101->U, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCUA(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	// The U register is up-to-date and will be used as this

	AL3101->A = mac1(AL3101->U, AL3101_Disasm->c, AL3101->A, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SCUB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	// The U register is up-to-date and will be used as this

	AL3101->A = mac1(AL3101->U, AL3101_Disasm->c, AL3101->B, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _DAC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->U = AL3101->A;							// update the U register with the last value used

	AL3101->A = mac1(AL3101->A, AL3101_Disasm->c, AL3101_Disasm->d, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _EXPB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->A = exp1(AL3101->B, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SEXPB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	AL3101->A = exp1(AL3101->B, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LOGB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 temp32;

	if (AL3101->B < 0) temp32 = AL3101->B & 0x07FFFFFF;
	else if (AL3101->B == 0)
	{
		AL3101->A = -113246208;
		AL3101->n = 0;
		AL3101->z = 1;
		AL3101->v = 0;
		return;
	}
	else temp32 = AL3101->B;
	
	AL3101->A = log1(temp32, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _SLOGB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 temp32;

	AL3101->Data[AL3101_Disasm->m] = AL3101->A;		// memory transfer

	if (AL3101->B < 0) temp32 = AL3101->B & 0x07FFFFFF;
	else if (AL3101->B == 0)
	{
		AL3101->A = -113246208;
		AL3101->n = 0;
		AL3101->z = 1;
		AL3101->v = 0;
		return;
	}
	else temp32 = AL3101->B;
	
	AL3101->A = log1(temp32, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _MMC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used
	m24 = AL3101->U & 0xFFFFFFF0;
	
	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _LMMC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->B = AL3101->Data[AL3101_Disasm->m];		// memory transfer

	AL3101->U = AL3101->B;							// update the U register with the last value used
	m24 = AL3101->U & 0xFFFFFFF0;

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _XMMC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 m24;

	AL3101->B = AL3101->A;							// register transfer

	AL3101->U = AL3101->Data[AL3101_Disasm->m];			// update the U register with the last value used
	m24 = AL3101->U & 0xFFFFFFF0;

	AL3101->A = mac1(AL3101->U, m24, AL3101_Disasm->c, AL3101->rounding, &(AL3101->n), &(AL3101->z), &(AL3101->v));

	return;
}


void _ANDB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
        AL3101->U = 0;
	AL3101->A = AL3101->A & AL3101->B;

	// If flags have to be affected, affect them here

	return;
}


void _ANDC(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
        AL3101->U = 0;
	AL3101->A = AL3101->A & AL3101_Disasm->c;

	// If flags have to be affected, affect them here

	return;
}


void _C(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
//	INT32 temp32;
	INT16 condition;

	/* The accumulator will not be tested however the specification tells it should be done.
	   Indeed, comparing to the hardware, it seems that overflow is on when abs(value) > 8.0
	   As a consequence, the flags are supposed to be set by previous instruction and
	   The accumulator will not be checked
	// Test the accumulator
	temp32 = AL3101->A;							// Load the accumulator
	AL3101->v = 0;								// By defaut, there's no overflow
	if (temp32 > 0)								// test depending on the sign bit
	{											// if result is positive
		if (temp32 > 0x00FFFFFF) AL3101->v = 1;		// if A > 1.0 => set overflow
		AL3101->n = 0;
		AL3101->z = 0;
	}
	else if (temp32 < 0)
	{											// if result is negative
		if (temp32 < 0xFF000000) AL3101->v = 1;		// if A < -1.0 => set overflow
		AL3101->n = 1;
		AL3101->z = 0;
	}
	else
	{											// if result if null
		AL3101->n = 0;
		AL3101->z = 1;
	}*/

	// make the test depending on the condition defined in the opcode
	condition = -1;
	switch(AL3101_Disasm->m)					// the condition type is stored in m
	{
	case 0x00:		// !Z
		if (!AL3101->z) condition = 0;
		break;
	case 0x01:		// Z
		if (AL3101->z) condition = 0;
		break;
	case 0x02:		// !V
		if (!AL3101->v) condition = 0;
		break;
	case 0x03:		// V
		if (AL3101->v) condition = 0;
		break;
	case 0x04:		// N
		if (AL3101->n) condition = 0;
		break;
	case 0x05:		// !N
		if (!AL3101->n) condition = 0;
		break;
	case 0x06:		// VN
		if (AL3101->v & AL3101->n) condition = 0;
		break;
	case 0x07:		// V!N
		if (AL3101->v & !AL3101->n) condition = 0;
		break;
	case 0x08:		// !VN
		if (!AL3101->v & AL3101->n) condition = 0;
		break;
	case 0x09:		// !V!N
		if (!AL3101->v & !AL3101->n) condition = 0;
		break;
	case 0x0A:		// !V!N!Z
		if (!AL3101->v & !AL3101->n & !AL3101->z) condition = 0;
		break;
	case 0x0B:		// !N!Z
		if (!AL3101->n & !AL3101->z) condition = 0;
		break;
	case 0x0C:		// N+Z
		if (AL3101->n | AL3101->z) condition = 0;
		break;
	case 0x0D:		// !V(N+Z)
		if (!AL3101->v & (AL3101->n | AL3101->z)) condition = 0;
		break;
	case 0x0E:		// Always 0
		condition = 0;
		break;
	case 0x0F:		// VZ
		if (AL3101->v & AL3101->z) condition = 0;
		break;
	}

	// if the condition is 0 then load the accumulator with the constant otherwise with 0
	if (condition == 0) AL3101->A = AL3101_Disasm->c;
	else AL3101->A = 0;

	return;
}


void _LINDF(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 temp32;

	temp32 = AL3101->B & 0x000003FF;		// keep only the bits 0-9 of the B register
	temp32 = temp32 & AL3101_Disasm->c;		// AND the B[0:9] and the constant
	temp32 = AL3101_Disasm->m + temp32;		// Add is to the memory pointer
	if (temp32 > 0x400) temp32 -= 0x400;	// Modulo 0x400

	AL3101->A = AL3101->Data[temp32];		// Load the memory element to the accumulator
	
	return;
}


void _LINDI(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 temp32;

	temp32 = AL3101->B & 0x003FF000;		// keep only the bits 12-21 of the B register
	temp32 = temp32 >> 12;					// remove the bits 0-11 of the B register
	temp32 = temp32 & AL3101_Disasm->c;		// AND the B[0:9] and the constant
	temp32 = AL3101_Disasm->m + temp32;		// Add is to the memory pointer
	if (temp32 > 0x400) temp32 -= 0x400;	// Modulo 0x400

	AL3101->A = AL3101->Data[temp32];		// Load the memory element to the accumulator
	
	return;
}


void _SINDF(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 temp32;

	temp32 = AL3101->B & 0x000003FF;		// keep only the bits 0-9 of the B register
	temp32 = temp32 & AL3101_Disasm->c;		// AND the B[0:9] and the constant
	temp32 = AL3101_Disasm->m + temp32;		// Add is to the memory pointer
	if (temp32 > 0x400) temp32 -= 0x400;	// Modulo 0x400

	AL3101->Data[temp32] = AL3101->A;		// Load the memory element to the accumulator
	
	return;
}


void _SINDI(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	INT32 temp32;

	temp32 = AL3101->B & 0x003FF000;		// keep only the bits 12-21 of the B register
	temp32 = temp32 >> 12;					// remove the bits 0-11 of the B register
	temp32 = temp32 & AL3101_Disasm->c;		// AND the B[0:9] and the constant
	temp32 = AL3101_Disasm->m + temp32;		// Add is to the memory pointer
	if (temp32 > 0x400) temp32 -= 0x400;	// Modulo 0x400

	AL3101->Data[temp32] = AL3101->A;		// Load the memory element to the accumulator
	
	return;
}


void _SKIP(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
//	INT32 temp32;
	INT16 condition;

	/* The accumulator will not be tested however the specification tells it should be done.
	   Indeed, comparing to the hardware, it seems that overflow is on when abs(value) > 8.0
	   As a consequence, the flags are supposed to be set by previous instruction and
	   The accumulator will not be checked
	// Test the accumulator
	temp32 = AL3101->A;							// Load the accumulator
	AL3101->v = 0;								// By defaut, there's no overflow
	if (temp32 > 0)								// test depending on the sign bit
	{											// if result is positive
		if (temp32 > 0x00FFFFFF) AL3101->v = 1;		// if A > 1.0 => set overflow
		AL3101->n = 0;
		AL3101->z = 0;
	}
	else if (temp32 < 0)
	{											// if result is negative
		if (temp32 < 0xFF000000) AL3101->v = 1;		// if A < -1.0 => set overflow
		AL3101->n = 1;
		AL3101->z = 0;
	}
	else
	{											// if result if null
		AL3101->n = 0;
		AL3101->z = 1;
	}*/

	// make the test depending on the condition defined in the opcode
	condition = -1;
	switch(AL3101_Disasm->m)					// the condition type is stored in m
	{
	case 0x00:		// !Z
		if (!AL3101->z) condition = 0;
		break;
	case 0x01:		// Z
		if (AL3101->z) condition = 0;
		break;
	case 0x02:		// !V
		if (!AL3101->v) condition = 0;
		break;
	case 0x03:		// V
		if (AL3101->v) condition = 0;
		break;
	case 0x04:		// N
		if (AL3101->n) condition = 0;
		break;
	case 0x05:		// !N
		if (!AL3101->n) condition = 0;
		break;
	case 0x06:		// VN
		if (AL3101->v & AL3101->n) condition = 0;
		break;
	case 0x07:		// V!N
		if (AL3101->v & !AL3101->n) condition = 0;
		break;
	case 0x08:		// !VN
		if (!AL3101->v & AL3101->n) condition = 0;
		break;
	case 0x09:		// !V!N
		if (!AL3101->v & !AL3101->n) condition = 0;
		break;
	case 0x0A:		// !V!N!Z
		if (!AL3101->v & !AL3101->n & !AL3101->z) condition = 0;
		break;
	case 0x0B:		// !N!Z
		if (!AL3101->n & !AL3101->z) condition = 0;
		break;
	case 0x0C:		// N+Z
		if (AL3101->n | AL3101->z) condition = 0;
		break;
	case 0x0D:		// !V(N+Z)
		if (!AL3101->v & (AL3101->n | AL3101->z)) condition = 0;
		break;
	case 0x0E:		// Always 0
		condition = 0;
		break;
	case 0x0F:		// VZ
		if (AL3101->v & AL3101->z) condition = 0;
		break;
	}

	// if the condition is 0 then add "n" to the PC counter
	if (condition == 0) AL3101->PC += AL3101_Disasm->c;

	return;
}


void _ADDB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	UINT32 TempA, TempB;

	TempA = (AL3101->A & (INT32)0xFFFFF000) >> 12;		// we have to mask with 4 highest bits as well to keep the sign extension
	TempB = (AL3101->B & (INT32)0xFFFFF000) >> 12;		// the 12 lowest bits are set to zero

	AL3101->U = AL3101->B & (UINT32)0xFFFF000;
	AL3101->A = TempA + TempB;

	// re-align the result
	AL3101->A = AL3101->A << 12;

	// Setting of the overflow bit  will be done here
	AL3101->v = 0;
	if (AL3101->A > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		AL3101->n = 0;
		if (AL3101->A > (INT32)0x07FFF000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x08000000) == (UINT32)0x08000000)
			{
				AL3101->n = 1;
				AL3101->A = AL3101->A | (UINT32)0xF0000000;
			}
		}
		AL3101->z = 0;
	}
	else if (AL3101->A < (INT32)0)
	{				// if result is negative
		AL3101->n = 1;
		if (AL3101->A < (INT32)0xF8000000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x08000000) != (UINT32)0x08000000)
			{
				AL3101->n = 0;
				AL3101->A = AL3101->A & (UINT32)0x0FFFFFFF;
			}
		}
		AL3101->z = 0;
	}
	else
	{
		AL3101->n = 0;
		AL3101->z = 1;
	}

	return;
}


void _SUBB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	UINT32 TempA, TempB;

	TempA = (AL3101->A & (INT32)0xFFFFF000) >> 12;
	TempB = (AL3101->B & (INT32)0xFFFFF000) >> 12;

	AL3101->U = AL3101->B & (UINT32)0xFFFF000;
	AL3101->A = TempA - TempB;

	// re-align the result
	AL3101->A = AL3101->A << 12;

	// Setting of the overflow bit  will be done here
	AL3101->v = 0;
	if (AL3101->A > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		AL3101->n = 0;
		if (AL3101->A > (INT32)0x07FFF000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x08000000) == (UINT32)0x08000000)
			{
				AL3101->n = 1;
				AL3101->A = AL3101->A | (UINT32)0xF0000000;
			}
		}
		AL3101->z = 0;
	}
	else if (AL3101->A < (INT32)0)
	{				// if result is negative
		AL3101->n = 1;
		if (AL3101->A < (INT32)0xF8000000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x08000000) != (UINT32)0x08000000)
			{
				AL3101->n = 0;
				AL3101->A = AL3101->A & (UINT32)0x0FFFFFFF;
			}
		}
		AL3101->z = 0;
	}
	else
	{
		AL3101->n = 0;
		AL3101->z = 1;
	}

	return;
}


void _MLTB(AL3101_struct *AL3101, AL3101_Disasm_struct *AL3101_Disasm)
{
	UINT32 TempA, TempB;

	AL3101->v = 0;
	AL3101->z = 0;
	AL3101->n = 0;

	// Prepare the data
	TempA = AL3101->A >> 12;
	TempB = AL3101->B >> 12;
	AL3101->U = AL3101->B & (UINT32)0xFFFF000;

	// COMPUTE MULTIPLICATION
	AL3101->A = (TempA * TempB);

	// Setting of the overflow bit  will be done here
	AL3101->v = 0;
	if (AL3101->A > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		AL3101->n = 0;
		if (AL3101->A > (INT32)0x00007FFF)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x00008000) == (UINT32)0x00008000)
			{
				AL3101->n = 1;
				AL3101->A = AL3101->A | (UINT32)0xFFFF0000;
			}
		}
		AL3101->z = 0;
	}
	else if (AL3101->A < (INT32)0)
	{				// if result is negative
		AL3101->n = 1;
		if (AL3101->A < (INT32)0xFFFF8000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x00008000) != (UINT32)0x00008000)
			{
				AL3101->n = 0;
				AL3101->A = AL3101->A & (UINT32)0x0000FFFF;
			}
		}
		AL3101->z = 0;
	}

	// re-align the result
	AL3101->A = AL3101->A << 12;

	// let's finally check the zero bit 
	if (AL3101->A == 0)
	{
		AL3101->n = 0;
		AL3101->z = 1;
	}

	return;

/*	UINT32 TempA, TempB, FullResult;

	AL3101->v = 0;
	AL3101->z = 0;
	AL3101->n = 0;

	// Prepare the data
	TempA = AL3101->A >> 12;
	TempB = AL3101->B >> 12;
	AL3101->U = AL3101->B & (UINT32)0xFFFF000;

	// COMPUTE MULTIPLICATION
	FullResult = (TempA * TempB);

	// re-align the result
	AL3101->A = FullResult << 12;

	// Setting of the overflow bit  will be done here
	AL3101->v = 0;
	if (AL3101->A > (INT32)0)	// done depending on the sign bit
	{				// if result is positive
		AL3101->n = 0;
		if (AL3101->A > (INT32)0x07FFF000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x08000000) == (UINT32)0x08000000)
			{
				AL3101->n = 1;
				AL3101->A = AL3101->A | (UINT32)0xF0000000;
			}
		}
		AL3101->z = 0;
	}
	else if (AL3101->A < (INT32)0)
	{				// if result is negative
		AL3101->n = 1;
		if (AL3101->A < (INT32)0xF8000000)
		{
			AL3101->v = 1;
			if ((AL3101->A & (UINT32)0x08000000) != (UINT32)0x08000000)
			{
				AL3101->n = 0;
				AL3101->A = AL3101->A & (UINT32)0x0FFFFFFF;
			}
		}
		AL3101->z = 0;
	}
	else
	{
		AL3101->n = 0;
		AL3101->z = 1;
	}

	return;*/
}
