#ifndef __AL3101STRUCT_H__
#define __AL3101STRUCT_H__

#include "../../common/1K_types.h"
#include "AL3101DbgFiles.h"

#define MAX_IO 1024			// Absolute number of inputs allowed
#define MAX_TOOL 1024		// Absolute number of outputs allowed
#define CMD_MAX_LEN 256

/* addresses over 1023 correspond to Input / output :
**   $400 - $40F : DIROUT1 -> DIROUT8 (W)
**   $420 - $42F : DIRIN1  -> DIRIN8  (R)
**   $449        : DIRIN              (R)
**   $418        : LOGB               (R)
**   $41B        : NullOUT        
**   $410 - $417 : SerIO              (RW)
** For clarity and due to hardware organization, we have splitted 
** the Serial IO into In and Out parts : the IN1 to IN8 addresses will
** correspond to Data[0x410 - 0x417] while an offset has been added
** to OUT1 - OUT8 which then corespond to Data[0x430 - 0x437]. 
** Of course, NOTHING changes in the source code !! This splitting is
** threated as a purely internal part of the silulator.
** In order to leave the simulation as fast as possible, the splitting 
** occurs during the assembling phase. All commands trying to WRITE to 
** addresses 0x410 to 0x417 will be moved to 0x430 to 0x437.
** The commands SINDI and SINDF act on addresses fixed only during 
** runtime. However, the resulting address is taken modulo 0x400. 
** Therefore, we do not have to worry about these ones
**
** Note that, as a consequence, the assembler used in this simulator
** can not be used as is in order to obtain an object code !
*/
typedef struct AL3101_structtag
{
  INT32 Data[1080];   
  INT32 A;
  INT32 B;
  INT32 U;
  INT32 rounding;
  INT32 n;
  INT32 z;
  INT32 v;
  INT32 PC;
  INT32 NumberOfLines;

  INT32 *Reg;
  INT32 *In;
  INT32 *Out;
  INT32 *DirectIn;
  INT32 *DirectOut;

  AL3101_structtag(){
	Reg		  = Data + 0x400;
    In        = Data + 0x410;
    Out       = Data + 0x430;
    DirectIn  = Data + 0x419;
    DirectOut = Data + 0x420;
  }
} AL3101_struct;

typedef struct AL3101_Disasm_structtag
{
  INT32  m;			// memory adress
  INT32  c;			// constant (aligned to S3.24 by default)
  INT32  d;			// constant (aligned to S3.24 by default)
  INT32  op;		// opcode  
  INT32	 ln;		// line number in the assembler file
  INT32	 s_index;	// index in the AL3101Table (for simulation call)
  UINT32 lObj;		// Object code (binary): bits  0 to 31 
  UINT32 wObj;		//                       bits 32 to 40
  CHAR  CmdStr[CMD_MAX_LEN]; // string with the assembler code.
  TMapItem *Map;        // preassembler addresses
  int NbMapItems;       // number of items in Map
} AL3101_Disasm_struct;

typedef struct{
	void *pInstance;
	INT32 (*f_Action) (void *pInst);
	INT32 (*f_Clean) (void *pInst);
	INT32 (*f_GetPosition) (void *, INT32 *);
	INT32 (*f_GetLength) (void *, INT32 *);
	INT32 (*f_SetLooping) (void *, INT32);
	CHAR CmdLine[CMD_MAX_LEN];
} IOTable;

typedef struct{
	void *pInstance;
	INT32 (*f_Read) (void *pInst);
	INT32 (*f_Write) (void *pInst);
	INT32 (*f_Clean) (void *pInst);
	INT32 (*f_GetPosition) (void *, INT32 *);
	INT32 (*f_GetLength) (void *, INT32 *);
	CHAR CmdLine[CMD_MAX_LEN];
} ToolTable;


#endif
