/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : June 2001                                       */
/* Project Version: v0.1                                            */
/* FileName       : Commands.c                                      */
/* File Version   : v0.1                                            */
/* Author         : Fabian Duchesne                                 */
/* Last Modif     : September 2001                                  */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*                  G L O B A L   I N C L U D E  					*/
/********************************************************************/
#include <stdio.h>
#include <string.h>
//#include <ctype.h>

/********************************************************************/
/*                  W X W I N     I N C L U D E						*/
/********************************************************************/
//#include <wx/wxprec.h>
#include <wx/wx.h>


/********************************************************************/
/*                   L O C A L   I N C L U D E						*/
/********************************************************************/
#include "./../MessageMacros.h"
#include "./../Simul_func/AL3101_Core.h"
#include "./../1K_func/AL3101Table.h"
#include "Commands.h"
#include "./../Error.h"
#include "./../Debug.h" 
#include "./../HelperFunctions.h"
#include "./../Simul_func/1KConfig.h"

/********************************************************************/
/*              C O N S T A N T S   D E F I N E S					*/
/********************************************************************/

/********************************************************************/
/*                G L O B A L   C O N S T A N T S					*/
/********************************************************************/
const CHAR* PROMPT_1KSM = "1KSM> ";
const INT32 MjrVersion = 0;
const INT32 MnrVersion = 1;

extern TMnemonicTable		MnemonicTable[];
extern AL3101SimCommand		AL3101SimCommandList[];
extern INT32				NumOfCommandsInList;

/********************************************************************/
/*                      P R O T O T Y P E S							*/
/********************************************************************/
void GetCommandString(CHAR *, CHAR *);
INT32 GetCommandStringFromScript(CHAR* Command, CHAR* Options, FILE *ScriptFile);

INT32 ExecuteCommand(INT32, CHAR*);
INT32 ExecuteScript(CHAR *ScriptFileName);

extern AL3101* pDspInstance;


/************************************************************/
/*                 M A I N    F O N C T I O N				*/
/************************************************************/
/************************************************************/
/* NAME       : main										*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
int main(int argc, char* argv[])
{
	CHAR Options[CMD_MAX_LEN];
	CHAR CommandStr[CMD_MAX_LEN];
	INT32 CommandID = -1;
	INT32 CommandNumInArray = -1;
	int ReturnCode = 0;
	
	
	OUTPUT_2("ALESIS AL3101/1KM SIMULATOR v%d.%d , AXORIS (c) Copyright\n", MjrVersion, MnrVersion);
	DBG_PRINT("In Main Function\n");

	// Initialize wxWindows library
    if ( !wxInitialize() )
    {
        ERROR("Failed to initialize the wxWindows library, aborting.");
		return 1;
    }

	// Initialize 1K DSP
	AL3101_Init(pDspInstance);

	// Load the configuration file
	if (Configuration_CheckIntegrity() != 0) WARNING("Config file not used, default values used instead.\n");
	else Configuration_Load();

	// Allocate the memory for the IOs, Signals, Traces and Tools tables
	pDspInstance->IO_Inputs = new IOTable[pDspInstance->Config.MaxNumberOfInputs];
	pDspInstance->IO_Outputs = new IOTable[pDspInstance->Config.MaxNumberOfOutputs];
	pDspInstance->Signals = new IOTable[pDspInstance->Config.MaxNumberOfSignals];
	pDspInstance->Traces = new IOTable[pDspInstance->Config.MaxNumberOfTraces];
	pDspInstance->Tools = new ToolTable[pDspInstance->Config.MaxNumberOfTools];
	if ((pDspInstance->IO_Inputs == NULL) || (pDspInstance->IO_Outputs == NULL) || (pDspInstance->Signals == NULL) || (pDspInstance->Traces == NULL) || (pDspInstance->Tools == NULL))
	{	// unable to allocate the memory for the IOs, ...
		ERROR("Unable to allocate memory for the IOs tables.\nExiting program.\n");
		ReturnCode = 2;
		goto Main_AfterLoop;
	}

	// Check if a script has to be executed
	if (argc > 1)
	{
		INT32 Mode = 0;
		CHAR TempStr[CMD_MAX_LEN];

		OUTPUT("Trying out program argument as script filename.\n");
		if (argc > 2)
		{
			String_ToLower_String(TempStr, argv[2]);
			if (strcmp(argv[2], "exit_if_error") == 0) Mode = 1;
		}
		INT32 TempReturnCode = ExecuteScript(argv[1]);
		if (Mode == 1)		// we must return if script is over
		{
			if ((TempReturnCode == 0) || (TempReturnCode == 1)) goto Main_AfterLoop;
			else
			{
				ReturnCode = 3;
				goto Main_AfterLoop;
			}
		}
		else				// we should not return except if "exit" is done within the script
		{
			if (TempReturnCode == 1) goto Main_AfterLoop;
		}
	}

	while (CommandID != 0)
	{
		//memset(CommandStr, 0, CMD_MAX_LEN);
		//memset(Options, 0, CMD_MAX_LEN);
		GetCommandString(CommandStr, Options);
		CommandNumInArray = GetCommandID(CommandStr, &CommandID);
		if (CommandNumInArray != NumOfCommandsInList) ExecuteCommand(CommandNumInArray, Options);
		else WARNING("Command not recognized.\n");
	}

Main_AfterLoop:
	// Save the configuration file
	Configuration_Save();

	// Uninitialize 1K DSP
	AL3101_Clean(pDspInstance);

	// Free the memory for the IOs, Signals, Traces and Tools tables
	delete[] pDspInstance->IO_Inputs;
	delete[] pDspInstance->IO_Outputs;
	delete[] pDspInstance->Signals;
	delete[] pDspInstance->Traces;
	delete[] pDspInstance->Tools;

	// Uninitialize wxWindows library
	wxUninitialize();

	DBG_PRINT("Out Main Function\n");

	return ReturnCode;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void GetCommandString(CHAR* Command, CHAR* Options)
{
	CHAR CommandLine[CMD_MAX_LEN];
	
	DBG_PRINT("In GetCommand\n");

	// print the prompt
	OUTPUT(PROMPT_1KSM);
	FLUSH_OUTPUT;
	// get the command line from the console and wait here
	fgets(CommandLine, CMD_MAX_LEN, stdin);
	// extract the command from the command line
	GetNextWord(Command, CommandLine);
	// make it lower case to be user-friendly
	String_ToLower_String(Command, Command);

	strcpy(Options, CommandLine);

	DBG_PRINT("Out GetCommand\n");
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 GetCommandStringFromScript(CHAR* Command, CHAR* Options, FILE *ScriptFile)
{
	CHAR TempStr[CMD_MAX_LEN];
	
	DBG_PRINT("In GetCommandFromScript\n");

	// get the command line from the script file
	if (fgets(TempStr, CMD_MAX_LEN, ScriptFile) == NULL) return 2;
	if ((strcmp(TempStr, "\n") == 0) || (strcmp(TempStr, "") == 0) || (TempStr == NULL)) return 1;
	OUTPUT_2("\r%s%s\n",PROMPT_1KSM, TempStr);
	FLUSH_OUTPUT;

	// extract the command from the command line
	GetNextWord(Command, TempStr);
	
	// make it lower case to be user-friendly
	String_ToLower_String(Command, Command);
	
	// copy the rest of the string in the option part
	strcpy(Options, TempStr);

	DBG_PRINT("Out GetCommandFromScript\n");

	return 0;
}

/****************************************************************/
/* FUNCTION NAME :												*/                   
/* DESCRIPTION   :												*/
/* INPUT         :												*/
/* OUTPUT        :												*/
/****************************************************************/
INT32 ExecuteCommand(INT32 ii, CHAR *OptStr)
{
	int ReturnCode;

	DBG_PRINT("In ExecuteCommand\n");
	
	ReturnCode = (AL3101SimCommandList[ii].function)(OptStr);

	DBG_PRINT("Out ExecuteCommand\n");

	return ReturnCode;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 ExecuteScript(CHAR *ScriptFileName)
{
	FILE		*ScriptFile;
	CHAR		Options[CMD_MAX_LEN];
	CHAR		CommandStr[CMD_MAX_LEN];
	CHAR		ErrorStr[CMD_MAX_LEN];
	INT32		CommandID = -1;
	INT32		CommandNumInArray = -1;
	
	DBG_PRINT("In ExecuteScript\n");

	// open the script file
	ScriptFile = fopen(ScriptFileName, "rt");
	if (ScriptFile == NULL)
	{
		ERROR_1("Unable to open script file: %s.\n", ScriptFileName);
		return -1;
	}

	while(CommandID != 0)
	{
		//memset(CommandStr, 0, CMD_MAX_LEN);
		//memset(Options, 0, CMD_MAX_LEN);
		if (GetCommandStringFromScript(CommandStr, Options, ScriptFile) != 0) break;
		strcpy(ErrorStr, CommandStr);
		strcat(ErrorStr, Options);
		CommandNumInArray = GetCommandID(CommandStr, &CommandID);
		if (CommandNumInArray != NumOfCommandsInList) 
		{
			if ((ExecuteCommand(CommandNumInArray, Options) & 0x0000FFFF) != 0) // masking to remove the base and class errors
			{
				ERROR_1("Error occurred during script in command: %s\n", ErrorStr);
				return -1;
			}
		}
		else ERROR("Command not recognized.\n");
	}
	
	// closing the script file
	fclose(ScriptFile);

	if (CommandID == 0)
	{
		OUTPUT("User exited within script.\n\nExiting simulator.\n");
	
		DBG_PRINT("Out ExecuteScript\n");
		return 1;
	}


	DBG_PRINT("Out CmdLoad\n");
	return 0;
}
