/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : April 2002                                      */
/* Project Version: v0.1                                            */
/* FileName       : Config.c                                        */
/* File Version   : v0.1                                            */
/* Author         : Fabian Duchesne                                 */
/* Last Modif     : April 2002                                      */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*       G L O B A L   I N C L U D E								*/
/********************************************************************/
#include <stdio.h>

/********************************************************************/
/*       W X   W i n   I N C L U D E								*/
/********************************************************************/
//#include <wx/wx.h>
#include <wx/config.h>


/********************************************************************/
/*       L O C A L   I N C L U D E									*/
/********************************************************************/
#include "./../MessageMacros.h"
#include "AL3101_Core.h"
#include "./../1K_func/AL3101Table.h"
#include "./../Debug.h"
#include "./../Error.h"
#include "./../HelperFunctions.h"
#include "1KConfig.h"

/********************************************************************/
/*       G L O B A L   V A R I A B L E S							*/
/********************************************************************/
extern const CHAR* PROMPT_1KSM;
extern AL3101* pDspInstance;


/********************************************************************/
/*       H E L P E R   F U N C   P R O T O T Y P E S				*/
/********************************************************************/
INT32 Configuration_Create();
INT32 LoadAliasFromConfig();
INT32 SaveAliasInConfig();
INT32 CmdSetAlias(CHAR* Options);




/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Configuration_CheckIntegrity()
{
	wxConfig *config;
	wxString str;
	long value;

	// Instanciate the config file and create the config file
	config = new wxConfig("VirtuAL3101", "Axoris Software");
	if (config == NULL)
	{
		ERROR("Unable to create the configuration file.\n");
		return -1;
	}

	// Check if config file is OK from appli and vendor name.
	str = config->GetAppName();
	if (str != "VirtuAL3101") goto Config_Configuration_CheckIntegrity_Error;
	str = config->GetVendorName();
	if (str != "Axoris Software") goto Config_Configuration_CheckIntegrity_Error;
	// Check if the config file version is the good one
	if (!config->Read("ConfigFileVersion", &value)) goto Config_Configuration_CheckIntegrity_Error;
	if (value != 1)
	{
		ERROR("Invalid config file version.\n");
		goto Config_Configuration_CheckIntegrity_Error;
	}
	// Check if the other parameters are there
	if (!config->Exists("/Simulation/TrueCycleSimulation")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Display/ShowDisasmWhenStepping")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Display/DefaultDisplayTypeInShow")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Config/MaxNumberOfInputs")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Config/MaxNumberOfOutputs")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Config/MaxNumberOfSignals")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Config/MaxNumberOfTraces")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Config/MaxNumberOfTools")) goto Config_Configuration_CheckIntegrity_Error;
	if (!config->Exists("/Simulation/DefaultResetType")) goto Config_Configuration_CheckIntegrity_Error;

	delete config;
	return 0;


Config_Configuration_CheckIntegrity_Error:
	ERROR("Configuration file doesn't exist or is invalid.\n");
	delete config;
	// create the new configuration
	if (Configuration_Create() == 0) return 0;
	else return -1;
}


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Configuration_Create()
{
	wxConfig *config;

	OUTPUT("Creating a new configuration file.\n");

	// Instanciate the config file and create the config file
	config = new wxConfig("VirtuAL3101", "Axoris Software");
	if (config == NULL) if (!config->Create()) goto Config_Configuration_Create_Error;
	
	// delete the configuration now it's created
	delete config;
	// save new values in this configuration file
	if (Configuration_Save() != 0) return -1;

	return 0;


Config_Configuration_Create_Error:
	ERROR("Unable to create the configuration file.\n");
	delete config;
	return -1;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Configuration_Load()
{
	wxConfig *config;
	long value;

	// Instanciate the config file and create the config file
	config = new wxConfig("VirtuAL3101", "Axoris Software");
	if (config == NULL)
	{
		ERROR("Unable to create the configuration file.\n");
		return -1;
	}

	// Load the "true cycle simulation" parameter
	if (!config->Read("/Simulation/TrueCycleSimulation", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.TrueCycleSimulation = (INT32)value;
	// Load the "show disasm when stepping" parameter
	if (!config->Read("/Display/ShowDisasmWhenStepping", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.ShowDisasmWhenStepping = (INT32)value;
	// Load the "default display type in show" parameter
	if (!config->Read("/Display/DefaultDisplayTypeInShow", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.DefaultDisplayTypeInShow = (INT32)value;
	// Load the "Maximal number of inputs" parameter
	if (!config->Read("/Config/MaxNumberOfInputs", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.MaxNumberOfInputs = (INT32)value;
	// Load the "Maximal number of outputs" parameter
	if (!config->Read("/Config/MaxNumberOfOutputs", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.MaxNumberOfOutputs = (INT32)value;
	// Load the "Maximal number of signals" parameter
	if (!config->Read("/Config/MaxNumberOfSignals", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.MaxNumberOfSignals = (INT32)value;
	// Load the "Maximal number of tracess" parameter
	if (!config->Read("/Config/MaxNumberOfTraces", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.MaxNumberOfTraces = (INT32)value;
	// Load the "Maximal number of tools" parameter
	if (!config->Read("/Config/MaxNumberOfTools", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.MaxNumberOfTools = (INT32)value;
	// Load the "default reset type" parameter
	if (!config->Read("/Simulation/DefaultResetType", &value)) goto Config_Configuration_Read_Error;
	pDspInstance->Config.DefaultResetType = (INT32)value;

	if (LoadAliasFromConfig() != 0) goto Config_Configuration_Read_Error;

	delete config;
	return 0;


Config_Configuration_Read_Error:
	ERROR("Unable to read from the configuration file. Giving up...\n");
	delete config;
	return -1;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Configuration_Save()
{
	wxConfig *config;
	long value;

	// Instanciate the config file 
	config = new wxConfig("VirtuAL3101", "Axoris Software");
	if (config == NULL)
	{
		ERROR("Unable to create the configuration file.\n");
		return -1;
	}

	// Write the config file version number
	value = CONFIG_FILE_VERSION_NUMBER;
	if (!config->Write("ConfigFileVersion", value)) goto Config_Configuration_Save_Error;

	// Write the "true cycle simulation" parameter
	value = pDspInstance->Config.TrueCycleSimulation;
	if (!config->Write("/Simulation/TrueCycleSimulation", value)) goto Config_Configuration_Save_Error;
	// Write the "show disasm when stepping" parameter
	value = pDspInstance->Config.ShowDisasmWhenStepping;
	if (!config->Write("/Display/ShowDisasmWhenStepping", value)) goto Config_Configuration_Save_Error;
	// Write the "default display type in show" parameter
	value = pDspInstance->Config.DefaultDisplayTypeInShow;
	if (!config->Write("/Display/DefaultDisplayTypeInShow", value)) goto Config_Configuration_Save_Error;
	// Write the "Maximal number of inputs" parameter
	value = pDspInstance->Config.MaxNumberOfInputs;
	if (!config->Write("/Config/MaxNumberOfInputs", value)) goto Config_Configuration_Save_Error;
	// Write the "Maximal number of outputs" parameter
	value = pDspInstance->Config.MaxNumberOfOutputs;
	if (!config->Write("/Config/MaxNumberOfOutputs", value)) goto Config_Configuration_Save_Error;
	// Write the "Maximal number of signals" parameter
	value = pDspInstance->Config.MaxNumberOfSignals;
	if (!config->Write("/Config/MaxNumberOfSignals", value)) goto Config_Configuration_Save_Error;
	// Write the "Maximal number of traces" parameter
	value = pDspInstance->Config.MaxNumberOfTraces;
	if (!config->Write("/Config/MaxNumberOfTraces", value)) goto Config_Configuration_Save_Error;
	// Write the "Maximal number of tools" parameter
	value = pDspInstance->Config.MaxNumberOfTools;
	if (!config->Write("/Config/MaxNumberOfTools", value)) goto Config_Configuration_Save_Error;
	// Write the "default reset type" parameter
	value = pDspInstance->Config.DefaultResetType;
	if (!config->Write("/Simulation/DefaultResetType", value)) goto Config_Configuration_Save_Error;

	if (SaveAliasInConfig() != 0) goto Config_Configuration_Save_Error;

	delete config;
	return 0;


Config_Configuration_Save_Error:
	ERROR("Unable to write in the configuration file. Giving up...\n");
	delete config;
	return -1;
}

INT32 LoadAliasFromConfig()
{
	wxConfig *config;
	long GroupIndex;
	UINT32 NumberOfEntries;
	INT32 index;
	wxString OriginalPath;
	CHAR TempStr[CMD_MAX_LEN + CMD_MAX_LEN + 1];
	wxString str;
	wxString CommandString;

	// Instanciate the config file 
	config = new wxConfig("VirtuAL3101", "Axoris Software");
	if (config == NULL)
	{
		ERROR("Unable to create the configuration file.\n");
		return -1;
	}

	// Trying to check if Aliases are saved in the configuration
	if (config->HasGroup("Alias") == FALSE) goto Config_Configuration_LoadAlias_NoAlias;

	// Aliases are stored in the configuration, let's try to find the first one
	OriginalPath = config->GetPath();					// Change first the path to go in the alias part
	config->SetPath("Alias");
	NumberOfEntries = config->GetNumberOfEntries();		// Get the number of entries in this part

	config->GetFirstEntry(str, GroupIndex);				// Get the first entry in this part
	for (index = 0; index < (INT32)NumberOfEntries; index++)
	{
		if (!config->Read(str, &CommandString)) goto Config_Configuration_LoadAlias_Error;
		strcpy(TempStr, str.GetData());
		strcat(TempStr, " ");
		strcat(TempStr, CommandString.GetData());			// String for command prepared
		CmdSetAlias(TempStr);								// Call the standard command for creating aliases
		config->GetNextEntry(str, GroupIndex);				// Get the next entry in this part
	}

	config->SetPath(*OriginalPath);						// Restore the original path

Config_Configuration_LoadAlias_NoAlias:		// This is not such a big problem, there are no aliases, that's all.
	delete config;
	return 0;


Config_Configuration_LoadAlias_Error:
	ERROR("Unable to read from the configuration file. Giving up...\n");
 	delete config;
	return -1;
}

INT32 SaveAliasInConfig()
{
	wxConfig *config;
	CHAR TempStr[CMD_MAX_LEN + 7];
	INT32 AliasCount;
	INT32 index, AliasIndex = 1;
	AL3101_CommandAlias		 *Alias;
	CommandAlias_List::Node  *Node;

	// Instanciate the config file 
	config = new wxConfig("VirtuAL3101", "Axoris Software");
	if (config == NULL)
	{
		ERROR("Unable to create the configuration file.\n");
		return -1;
	}

	// Delete the existing alias group if any
	if (config->Exists("/Alias")) config->DeleteGroup("Alias");

	// Look at the number of aliases defined
	AliasCount = pDspInstance->Config.Alias.GetCount();
	// If 0, then no need to proceed
	if (AliasCount == 0) goto Config_Configuration_SaveAlias_ReturnOK;

	// Make a loop on the available aliases
	strcpy(TempStr, "/Alias/");
	for (index = 0; index < AliasCount; index++)
	{
		Node = pDspInstance->Config.Alias.Item(index);
		Alias = Node->GetData();
		strcpy(TempStr + 7, Alias->AliasString);
		if (!config->Write(TempStr, (const wxString&)Alias->CommandString)) goto Config_Configuration_SaveAlias_Error;
	}


Config_Configuration_SaveAlias_ReturnOK:
	delete config;
	return 0;


Config_Configuration_SaveAlias_Error:
	ERROR("Unable to save to the configuration file. Giving up...\n");
	delete config;
	return -1;
}