#include "CFNoiseGenerator.h"

#ifdef __CFNOISEGENERATOR_RANDOM__
CFNoiseGenerator::CFNoiseGenerator()
{
	randSeed = 22222;
}

/* Calculate pseudo-random 32 bit number based on linear congruential method. */ 
void CFNoiseGenerator::Process(float *output)
{ 
	float tempfloat;

	/* Change this for different random sequences. */ 
	randSeed = (randSeed * 196314165) + 907633515; 
	tempfloat = (float)randSeed - 2147483647.0f;
	*output = tempfloat / 4294967295.0f;
}
#endif


#ifdef __CFNOISEGENERATOR_GAUSSIAN__

#include "stdlib.h"			// for the rand function
#include "math.h"			// for the sqrt, log and cos functions
#define PI 3.1415926536f

void CFNoiseGenerator::Process(float *output)
{ 
	float R1 = (float) (rand() + 1) / (float) (RAND_MAX + 1); 
	float R2 = (float) rand() / (float) RAND_MAX; 

	float X = (float) sqrt( -2.0f * log( R1 )) * cos( 2.0f * PI * R2 )
		/ (float) sqrt( -2.0f * log( 1.0f / (float)(RAND_MAX + 1) )); 

	*output = X;
}
#endif



#ifdef __CFNOISEGENERATOR_COMPLEX__

#include "stdlib.h"			// for the rand function
#include "math.h"			// for the sqrt, log and cos functions
#define PI 3.1415926536f
#define ranf() ((float) rand() / (float) RAND_MAX) 

void CFNoiseGenerator::Process(float *output)
{ 

   static int pass = 0; 
   static float y2; 
   float x1, x2, w, y1; 

   if (pass) 
   { 
      y1 = y2; 
   } else  { 
      do { 
         x1 = 2.0f * ranf () - 1.0f; 
         x2 = 2.0f * ranf () - 1.0f; 
         w = x1 * x1 + x2 * x2; 
      } while (w >= 1.0f); 

      w = (float)sqrt (-2.0 * log (w) / w); 
      y1 = x1 * w; 
      y2 = x2 * w; 
   } 
   pass = !pass; 
	
   *output = y1;

//   return ( (y1 * s + (float) m)); 
}
#endif