/***********************************************************************************************
 *
 * Functions	: CFSinGenerator v1.0
 *
 * Members Functions:
 *
 *		1. Initialization
 *			void SinGenerator_Init(SinGeneratorStruct *pInst, float SamplingFrequency, float Frequency, float Amplitude; float InitialPhase)			
 *
 *		2. Set Functions
 *			void SinGenerator_SetSamplingFrequency(SinGeneratorStruct *pInst, float SamplingFrequency)
 *			void SinGenerator_SetFrequency(SinGeneratorStruct *pInst, float Frequency)
 *			void SinGenerator_SetAmplitude(SinGeneratorStruct *pInst, float Amplitude)
 *			void SinGenerator_SetInitialPhase(SinGeneratorStruct *pInst, float InitialPhase)
 *		
 *		3. Process Function
 *			void SinGenerator_Process(SinGeneratorStruct *pInst, float *pOut) 
 *          void SinGenerator_ProcessBlock(SinGeneratorStruct *pInst, float *pOut, INT32 BufLen)
 *
 ************************************************************************************************/
#include "CFSinGenerator.h"
#include "math.h"				// used by sin/cos 1st calc.


// Initialization
void CFSinGenerator::Init(float ParSamplingFrequency, float ParFrequency, float ParAmplitude, float ParInitialPhase)
{
	SamplingFrequency = ParSamplingFrequency;
	Frequency = ParFrequency;
	Amplitude = ParAmplitude;
	InitialPhase = ParInitialPhase;

	sinalpha = (float) sin(2.0f * 3.14159265358f * ParFrequency / ParSamplingFrequency);
	cosalpha = (float) cos(2.0f * 3.14159265358f * ParFrequency / ParSamplingFrequency);
	lastsin = (float)sin(ParInitialPhase);
	lastcos = (float)sin(ParInitialPhase + 1.57079632679f);
}																 


// Set Functions
void CFSinGenerator::SetSamplingFrequency(float ParSamplingFrequency)
{
	SamplingFrequency = ParSamplingFrequency;

	sinalpha = (float) sin(2.0f * 3.14159265358f * Frequency / SamplingFrequency);
	cosalpha = (float) cos(2.0f * 3.14159265358f * Frequency / SamplingFrequency);
}

void CFSinGenerator::SetFrequency(float ParFrequency)
{
	Frequency = ParFrequency;

	sinalpha = (float) sin(2.0f * 3.14159265358f * Frequency / SamplingFrequency);
	cosalpha = (float) cos(2.0f * 3.14159265358f * Frequency / SamplingFrequency);
}

void CFSinGenerator::SetAmplitude(float ParAmplitude)
{
	Amplitude = ParAmplitude;
}

void CFSinGenerator::SetInitialPhase(float ParInitialPhase, INT32 ResetCurrentPhase)
{
	InitialPhase = ParInitialPhase;

	if (ResetCurrentPhase != 0) Init(SamplingFrequency, Frequency, Amplitude, InitialPhase);
}

void CFSinGenerator::Reset()
{
	Init(SamplingFrequency, Frequency, Amplitude, InitialPhase);
}		

// Process Function
void CFSinGenerator::Process(float *pOut)
{
	float tempfloat;
	float oldsin;

	// writing in buffer
	*pOut = lastsin * Amplitude;

	// generation of the sine/cosine
	tempfloat = lastsin * cosalpha + lastcos * sinalpha;
	lastcos = lastcos * cosalpha - lastsin * sinalpha;
	oldsin = lastsin;
	lastsin = tempfloat;

	// do not do the Alesis like self-reset. it seems to be useless in float
	/*if (lastsin > 0)	// skip reset if new sin is < 0
	{
		if (oldsin < 0)	// skip reset if old sin is >= 0
			lastcos = 1.0f;
	}*/
	// perform some overflow testing.
	if (lastsin >  1.0f) lastsin =  1.0f;
	if (lastsin < -1.0f) lastsin = -1.0f;


}

void CFSinGenerator::ProcessBlock(float *pOut, INT32 BufLen)
{
	float tempfloat;
	INT32 index;

	for (index = 0; index < BufLen; index++)
	{
		// writing in buffer
		*pOut++ = lastsin * Amplitude;

		// generation of the sine/cosine
		tempfloat = lastsin * cosalpha + lastcos * sinalpha;
		lastcos = lastcos * cosalpha - lastsin * sinalpha;
		lastsin = tempfloat;
	
		// do not do the Alesis like self-reset. it seems to be useless in float
		/*if (lastsin > 0)	// skip reset if new sin is < 0
		{
			if (oldsin < 0)	// skip reset if old sin is >= 0
				lastcos = 1.0f;
		}*/

		// perform some overflow testing.
		if (lastsin >  1.0f) lastsin =  1.0f;
		if (lastsin < -1.0f) lastsin = -1.0f;
	}
}
