#include <math.h>

#include "FFTWindowing.h"

#ifdef WIN32
float M_PI = 3.1415926535f;
#endif

void HanningWindow(float *Out, int Length)
{
	int index;
	float value;

	for (index = 0; index < Length; index++)
	{
		value = 0.5f * (1.0f - (float)cos((2 * M_PI * index) / (Length - 1)));
		Out[index] = value;
	}
}

void ApplyHanningWindow(float *Out, float *In, int Length)
{
	int index;
	float value;

	for (index = 0; index < Length; index++)
	{
		value = 0.5f * (1.0f - (float)cos((2 * M_PI * index) / (Length - 1)));
		Out[index] = value * In[index];
	}
}

void BartlettWindow(float *Out, int Length)
{
	int index;
	int indexKnee;
	float value;

	indexKnee = Length / 2;		// Test (index <= (L-1)/ 2) equivalent to test (index < L/2)

	for (index = 0; index < indexKnee; index++)
	{
		value = 2.0f * index / (Length - 1.0f);
		Out[index] = value;
	}

	for (index = indexKnee; index < Length; index++)
	{
		value = 2.0f - 2.0f * index / (Length - 1.0f);
		Out[index] = value;
	}
}

void ApplyBartlettWindow(float *Out, float *In, int Length)
{
	int index;
	int indexKnee;
	float value;

	indexKnee = Length / 2;		// Test (index <= (L-1)/ 2) equivalent to test (index < L/2)

	for (index = 0; index < indexKnee; index++)
	{
		value = 2.0f * index / (Length - 1.0f);
		Out[index] = value * In[index];
	}

	for (index = indexKnee; index < Length; index++)
	{
		value = 2.0f - 2.0f * index / (Length - 1.0f);
		Out[index] = value * In[index];
	}
}
