// Gen_SIN.cpp: implementation of the White noise generator functions.
//
/////////////////////////////////////////////////////////////////////////////////

#include "../MessageMacros.h"		// defines the macros for displaying macros
#include "Gen_WNO.h"
#include "Gen_WNO_Internal.h"

#include <stdlib.h>		// for the malloc function
#include <string.h>		// for the string routines
#include <math.h>		// for the log function


//////////////////////////////////////////////////////////////////////
// init & clean functions
//////////////////////////////////////////////////////////////////////
INT32 Gen_WNO_Init(void **pInstance, Generator_WhiteNoiseCfg *cfg, INT32 *InputMemory)
{
	Gen_WNO_Struct *pInst;

	// Allocate the space for the instance structure
	pInst = new Gen_WNO_Struct;
	if (pInst == NULL) return -1; //TOOL_CANNOT_ALLOCATE_INSTANCE
	*pInstance = (void *)pInst;

	// Initialize parameter related variables
	pInst->Amplitude = cfg->Amplitude;
	pInst->Offset = cfg->Offset;

	// Initialize memory related variables
	pInst->InputMemory = InputMemory;

	return FILEIO_NOERROR;	// NO_ERROR
}

INT32 Gen_WNO_Clean(void *pInstance)
{
	Gen_WNO_Struct *pInst = (Gen_WNO_Struct *)pInstance;

	// freeing the instance structure
	if (pInst != NULL) delete pInst;

	return FILEIO_NOERROR;	// NO_ERROR
}


//////////////////////////////////////////////////////////////////////
// "processing" functions
//////////////////////////////////////////////////////////////////////
INT32 Gen_WNO_ReadSample(void *pInstance)
{
	float NoiseValue;
	INT32 NewSample;

	Gen_WNO_Struct *pInst = (Gen_WNO_Struct *)pInstance;

	// Generate new sample
	pInst->NoiseGenerator.Process(&NoiseValue);		// Generate the new noise value
	NoiseValue = NoiseValue * pInst->Amplitude;		// Apply the current amplitude
	NoiseValue = NoiseValue + pInst->Offset;		// Apply the offset
	NewSample = (INT32)(NoiseValue * 16777216.0f);	// transform to S3.24
	*(pInst->InputMemory) = NewSample;				// copy sine value to memory

	return FILEIO_NOERROR;
}



//////////////////////////////////////////////////////////////////////
// file-like function(s)
//////////////////////////////////////////////////////////////////////
INT32 Gen_WNO_GetPosition(void *pInstance, INT32 *CurrentPosition)
{
	Gen_WNO_Struct *pInst = (Gen_WNO_Struct *)pInstance;

	*CurrentPosition = -1;

	return FILEIO_NOERROR;	//TOOL_NOERROR
}

INT32 Gen_WNO_GetFileLength(void *pInstance, INT32 *FileSize)
{
	Gen_WNO_Struct *pInst = (Gen_WNO_Struct *)pInstance;

	*FileSize = -1;
	
	return FILEIO_NOERROR; //TOOL_NOERROR;
}



//////////////////////////////////////////////////////////////////////
// control function(s)
//////////////////////////////////////////////////////////////////////
INT32 Gen_WNO_SetLoopingMode(void *pInstance, INT32 Looping)
{
	return FILEIO_NOERROR;
}



/*//////////////////////////////////////////////////////////////////////
// "text" function(s)
//////////////////////////////////////////////////////////////////////
void FileIO_PCM_GetLastErrorString(CHAR *str, INT32 LastErrorCode, INT32 MaxNumberOfCharacters)
{
	if ((LastErrorCode & FILEIO_BASE_ERR) == FILEIO_BASE_ERR)		// check if the error is a FileIO error
	{
		if ((LastErrorCode & FILEIO_GEN_BASE_ERR) == FILEIO_GEN_BASE_ERR)
		{
			FileIO_GetLastErrorString(str, LastErrorCode, MaxNumberOfCharacters);
			return;
		}
		else if ((LastErrorCode & FILEIO_PCM_BASE_ERR) == FILEIO_PCM_BASE_ERR)
		{
			// do nothing here, wait for the switch below
		}
		else
		{
			strncpy(str, "Error type not known", MaxNumberOfCharacters - 1);
		}
	}
	else															// this error is not a FileIO error
	{
		strncpy(str, "Not a FileIO error", MaxNumberOfCharacters - 1);
		return;
	}


	switch(LastErrorCode)
	{
		case FILEIO_PCM_NOERROR:
			strncpy(str, "PCM: No error occurred", MaxNumberOfCharacters - 1);
			break;
		default:
			strncpy(str, "PCM: Error not known", MaxNumberOfCharacters - 1);
			break;
	}
	return;
}



void FileIO_PCM_GetHelp(CHAR *str, INT32 LineNumber, INT32 MaxNumberOfCharacters)
{
	switch(LineNumber)
	{
		case 0:
			strncpy(str, "PCM files are read by these functions.", MaxNumberOfCharacters - 1);
			break;
		case 1:
			strncpy(str, "PCM files are raw data encoded in Little Endian format i.e. LSB,MSB", MaxNumberOfCharacters - 1);
			break;
		case 2:
			strncpy(str, "There's no support for 8 bits files and for more than 4GB files.", MaxNumberOfCharacters - 1);
			break;
		default:
			strncpy(str, "PCM file functions", MaxNumberOfCharacters - 1);
	}
}
*/
