// Tool_AmplitudeSweep.h: interface for the AmplitudeSweep Tool functions.
//
///////////////////////////////////////////////////////////////////////////

#ifndef __AL3101__TOOL_AMPSWEEP__H__
#define __AL3101__TOOL_AMPSWEEP__H__

#include <stdio.h>							// for the file IO
#include "../../common/1K_types.h"	// for the simulator types
//#include "FileIO_Generic.h"				// for the definition of the generic file IO stuff

// Level parameters structure. To be used at init time.
typedef struct Tool_AmplitudeSweep_LevelParameters_structtag
{
	float InitialLevel;
	float FinalLevel;
	float LevelIncrement;
} Tools_AmplitudeSweep_LevelParameters;

// Timing parameter structure. To be used at init time.
typedef struct Tool_AmplitudeSweep_TimingParameters_structtag
{
	float TransientTime;
	float MeasurementTime;
	float SamplingFrequency;
	float SineFrequency;
} Tools_AmplitudeSweep_TimingParameters;

// Results structure.
typedef struct Tool_AmplitudeSweep_Results_structtag
{
	float InputAmplitude;
	float OutputAmplitude;
	float Gain;
} Tool_AmplitudeSweep_Results;


//////////////////////////////////
// definition of the prototypes
//////////////////////////////////
// init & clean functions
INT32 Tool_AmplitudeSweep_Init(void **pInstance, Tool_AmplitudeSweep_Results *results, Tools_AmplitudeSweep_LevelParameters *LevelParams, Tools_AmplitudeSweep_TimingParameters *TimingParams, INT32 *InputMemory, INT32 *OutputMemory, CHAR *AmplitudeFileName, CHAR *GainFileName);
INT32 Tool_AmplitudeSweep_Clean(void *pInstance);

// "processing" functions
INT32 Tool_AmplitudeSweep_ReadSample(void *pInstance);
INT32 Tool_AmplitudeSweep_WriteSample(void *pInstance);

// file-like functions
INT32 Tool_AmplitudeSweep_GetPosition(void *pInstance, INT32 *CurrentPosition);
INT32 Tool_AmplitudeSweep_GetFileLength(void *pInstance, INT32 *FileSize);

// control functions

// text functions
//VOID FileIO_PCM_GetLastErrorString(CHAR *str, INT32 LastErrorCode, INT32 MaxNumberOfCharacters);
//VOID FileIO_PCM_GetHelp(CHAR *str, INT32 LineNumber, INT32 MaxNumberOfCharacters);

// helper functions
INT32 Tool_AmplitudeSweep_GetResultSize(Tools_AmplitudeSweep_LevelParameters *LevelParams, INT32 *ResultSize);
INT32 Tool_AmplitudeSweep_WriteAmplitudeResults(Tool_AmplitudeSweep_Results *results, INT32 ResultSize, CHAR *AmplitudeFileName);
INT32 Tool_AmplitudeSweep_WriteGainResults(Tool_AmplitudeSweep_Results *results, INT32 ResultSize, CHAR *GainFileName);
//INT32 WriteReport();

//////////////////////////////////
// defines PCM error messages
//////////////////////////////////
//#define FILEIO_PCM_NOERROR					(FILEIO_PCM_BASE_ERR +  0)


#endif // !defined(__AL3101__TOOL_SWEEPAMP__H__)

